--------------------------------------------------------------------------------
------------- Creaci de totes les taules del sistema. -------------------------
--------------------------------------------------------------------------------

BEGIN

--------------------------------------------------------------------------------
--- Creaci de la taula Estat --------------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE ESTAT(
  codi VARCHAR2(4 CHAR) CONSTRAINT PK_ESTAT PRIMARY KEY,
  nom VARCHAR2(50 CHAR) CONSTRAINT NN_ESTAT_nom NOT NULL,
  CONSTRAINT UN_ESTAT UNIQUE(nom)
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Regi --------------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE REGIO(
  codi VARCHAR2(20 CHAR),
  estat VARCHAR2(4 CHAR) CONSTRAINT FK_REGIO REFERENCES ESTAT(codi)
                         ON DELETE CASCADE,
  nom VARCHAR2(50 CHAR) CONSTRAINT NN_REGIO_nom NOT NULL,
  CONSTRAINT PK_REGIO PRIMARY KEY(codi, estat)
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Localitat ----------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE LOCALITAT(
  codi VARCHAR2(20 CHAR),
  regio VARCHAR2(20 CHAR),
  estat VARCHAR2(4 CHAR),
  nom VARCHAR2(50 CHAR) CONSTRAINT NN_LOCALITAT_nom NOT NULL,
  CONSTRAINT PK_LOCALITAT PRIMARY KEY (codi, regio, estat),
  CONSTRAINT FK_LOCALITAT  FOREIGN KEY (regio, estat) 
  REFERENCES REGIO(codi, estat) ON DELETE CASCADE
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Cens ---------------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE CENS(
  nom VARCHAR2(45 CHAR) CONSTRAINT PK_CENS PRIMARY KEY,
  ambit VARCHAR2(45 CHAR) CONSTRAINT NN_CENS_ambit NOT NULL,
  votants NUMERIC DEFAULT 0,
  CONSTRAINT CK_CENS_ambit CHECK (ambit IN (''Global'', ''Estatal'', ''Regional'', ''Local''))
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Cens Estatal -------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE CENS_ESTAT(
  nom VARCHAR2(45 CHAR) CONSTRAINT FK_1_CENS_ESTAT REFERENCES CENS(nom)
                          ON DELETE CASCADE
                        CONSTRAINT PK_CENS_ESTAT PRIMARY KEY,
  estat VARCHAR2(4 CHAR) CONSTRAINT NN_CENS_ESTAT_estat NOT NULL
                         CONSTRAINT FK_2_CENS_ESTAT REFERENCES ESTAT(codi),
  CONSTRAINT UN_CENS_ESTAT UNIQUE(estat)
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Cens Regional ------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE CENS_REGIO(
  nom VARCHAR2(45 CHAR) CONSTRAINT FK_1_CENS_REGIO REFERENCES CENS(nom)
                          ON DELETE CASCADE
                        CONSTRAINT PK_CENS_REGIO PRIMARY KEY,
  estat VARCHAR2(4 CHAR) CONSTRAINT NN_CENS_REGIO_estat NOT NULL,
  regio VARCHAR2(20 CHAR) CONSTRAINT NN_CENS_REGIO_regio NOT NULL,
  CONSTRAINT UN_CENS_REGIO UNIQUE(estat, regio),
  CONSTRAINT FK_2_CENS_REGIO FOREIGN KEY (estat, regio) 
                             REFERENCES REGIO(estat, codi)
)';

--------------------------------------------------------------------------------
--- Creci de la taula Cens Local ----------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE CENS_LOCAL(
  nom VARCHAR2(45 CHAR) CONSTRAINT FK_1_CENS_LOCAL REFERENCES CENS(nom)
                          ON DELETE CASCADE
                        CONSTRAINT PK_CENS_LOCAL PRIMARY KEY,
  estat VARCHAR2(4 CHAR) CONSTRAINT NN_CENS_LOCAL_estat NOT NULL,
  regio VARCHAR2(20 CHAR) CONSTRAINT NN_CENS_LOCAL_regio NOT NULL,
  localitat VARCHAR2(20 CHAR) CONSTRAINT NN_CENS_LOCAL_localitat NOT NULL,
  CONSTRAINT UN_CENS_LOCAL UNIQUE(estat, regio, localitat),
  CONSTRAINT FK_2_CENS_LOCAL  FOREIGN KEY (estat, regio, localitat) 
                              REFERENCES LOCALITAT(estat, regio, codi)
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Ciutad ------------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE CIUTADA(
  codi NUMERIC CONSTRAINT PK_CIUTADA PRIMARY KEY,
  edat DATE CONSTRAINT NN_CIUTADA_edat NOT NULL,
  localitat VARCHAR2(20 CHAR) CONSTRAINT NN_CIUTADA_localitat NOT NULL,
  regio VARCHAR2(20 CHAR) CONSTRAINT NN_CIUTADA_regio NOT NULL, 
  estat VARCHAR2(4 CHAR) CONSTRAINT NN_CIUTADA_estat NOT NULL,
  n_vot_part NUMERIC DEFAULT 0 CONSTRAINT NN_CIUTADA_n_vot_part NOT NULL,
  CONSTRAINT FK_CIUTADA FOREIGN KEY (localitat, regio, estat)
                        REFERENCES LOCALITAT (codi, regio, estat)
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Votaci ------------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE VOTACIO(
  titol VARCHAR2(50 CHAR) CONSTRAINT PK_VOTACIO PRIMARY KEY,
  data_publicacio DATE DEFAULT SYSDATE,
  data_obertura DATE,
  data_tancament DATE,
  president VARCHAR2(50 CHAR) CONSTRAINT NN_VOTACIO_president NOT NULL,
  descripcio VARCHAR2(300 CHAR) CONSTRAINT NN_VOTACIO_descripcio NOT NULL,
  estat VARCHAR2(20 CHAR) DEFAULT ''Edici''
                          CONSTRAINT NN_VOTACIO_estat NOT NULL,
  web VARCHAR2(150 CHAR),
  total_ciutadans NUMERIC DEFAULT 0,
  total_vots NUMERIC DEFAULT 0,
  participacio NUMERIC(5,2) DEFAULT 0,
  abstencio NUMERIC(5,2) DEFAULT 0,
  dif_opcions NUMERIC(5,2) DEFAULT 0,  
  CONSTRAINT CH_VOTACIO_obertura CHECK(data_obertura >= data_publicacio + 7),
  CONSTRAINT CH_VOTACIO_tancament CHECK(data_tancament >= data_obertura),
  CONSTRAINT CH_VOTACIO_estat CHECK(estat IN (''Edici'', ''Pendent'', ''Oberta'', ''Tancada''))
)';

--------------------------------------------------------------------------------
--- Creacio de la taula Inscripci ---------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE INSCRIPCIO(
  ciutada NUMERIC CONSTRAINT FK_1_INSCRIPCIO REFERENCES CIUTADA(codi)
                  ON DELETE CASCADE,
  cens VARCHAR2(45 CHAR) CONSTRAINT FK_2_INSCRIPCIO REFERENCES CENS(nom)
                  ON DELETE CASCADE,
  data_ins DATE DEFAULT SYSDATE,
  CONSTRAINT PK_INSCRIPCIO PRIMARY KEY(ciutada, cens)
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Assignaci ---------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE ASSIGNACIO(
  cens VARCHAR2(45 CHAR) CONSTRAINT FK_1_ASSIGNACIO REFERENCES CENS(nom),
  votacio VARCHAR2(50 CHAR) CONSTRAINT FK_2_ASSIGNACIO REFERENCES VOTACIO(titol)
                            ON DELETE CASCADE,
  data_assig DATE DEFAULT SYSDATE,
  votants NUMERIC DEFAULT 0,
  vots NUMERIC DEFAULT 0,
  participacio NUMERIC(5,2) DEFAULT 0,
  abstencio NUMERIC(5,2) DEFAULT 0,
  CONSTRAINT PK_ASSIGNACIO PRIMARY KEY(cens, votacio)
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Opci --------------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE OPCIO(
  codi NUMERIC CONSTRAINT PK_OPCIO PRIMARY KEY,
  votacio VARCHAR2(50 CHAR) CONSTRAINT FK_OPCIO REFERENCES VOTACIO(titol)
                            ON DELETE CASCADE
                            CONSTRAINT NN_OPCIO_votacio NOT NULL,
  descripcio VARCHAR2(200 CHAR) CONSTRAINT NN_OPCIO_descripcio NOT NULL,
  vots_absoluts NUMERIC DEFAULT 0,
  vots_relatius NUMERIC(5,2) DEFAULT 0,
  CONSTRAINT UN_OPCIO UNIQUE(votacio, descripcio)
)';
--- Seqncia i Disparador que dona el valor a OPCIO.codi
EXECUTE IMMEDIATE 'CREATE SEQUENCE sec_opcio INCREMENT BY 1 START WITH 1';

EXECUTE IMMEDIATE 'CREATE OR REPLACE TRIGGER insert_codi_opcio
  BEFORE INSERT ON OPCIO
  FOR EACH ROW
BEGIN
  SELECT sec_opcio.NEXTVAL INTO :NEW.codi
  FROM DUAL;
END insert_codi_opcio;';

--------------------------------------------------------------------------------
--- Creaci de la taula Vot ----------------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE VOT(
  votacio VARCHAR2(50 CHAR) CONSTRAINT FK_1_VOT REFERENCES VOTACIO(titol),
  ciutada NUMERIC CONSTRAINT FK_2_VOT REFERENCES CIUTADA(codi),
  opcio NUMERIC CONSTRAINT FK_3_VOT REFERENCES OPCIO(codi)
                CONSTRAINT NN_VOT_opcio NOT NULL,
  data_vot DATE DEFAULT SYSDATE,
  CONSTRAINT PK_VOT PRIMARY KEY(votacio, ciutada)
)';

--- Disparador de seguretat que actua quan s'intenta editar o esborrar un Vot
EXECUTE IMMEDIATE 'CREATE OR REPLACE TRIGGER no_edit_borr_vot
  BEFORE UPDATE OR DELETE ON VOT
  FOR EACH ROW
BEGIN
  raise_application_error(-20000, ''ERROR: GREU!!. No es permet editar ni esborrar cap vot registrat'');  
END no_edit_borr_vot;';

--------------------------------------------------------------------------------
--- Creaci de la taula Taula_Logs ---------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE TAULA_LOGS(
  codi NUMERIC CONSTRAINT PK_TAULA_LOGS PRIMARY KEY,
  data_crida DATE DEFAULT SYSDATE,
  nom_procediment VARCHAR2(40 CHAR) CONSTRAINT NN_TAULA_LOGS_nom_procediment NOT NULL,
  par_entrada VARCHAR2(500 CHAR) CONSTRAINT NN_TAULA_LOGS_par_entrada NOT NULL,
  par_sortida VARCHAR2(500 CHAR) CONSTRAINT NN_TAULA_LOGS_par_sortida NOT NULL
)';
--- Seqncia i Disparador que dona el valor a TAULA_LOGS.codi
EXECUTE IMMEDIATE 'CREATE SEQUENCE sec_taula_logs INCREMENT BY 1 START WITH 1';

EXECUTE IMMEDIATE 'CREATE OR REPLACE TRIGGER insert_codi_taula_logs
  BEFORE INSERT ON TAULA_LOGS
  FOR EACH ROW
BEGIN
  SELECT sec_taula_logs.NEXTVAL INTO :NEW.codi
  FROM DUAL;
END insert_codi_taula_logs;';

--------------------------------------------------------------------------------
--- Creaci de la taula Est_Cens_Any -------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE EST_CENS_ANY(
  anys NUMBER,
  cens VARCHAR2(45 CHAR) CONSTRAINT FK_EST_CENS_ANY REFERENCES CENS(nom),
  n_vot_asoc NUMBER DEFAULT 0,
  v_mig_part NUMBER(5,2) DEFAULT 0,
  CONSTRAINT PK_EST_CENS_ANY PRIMARY KEY(anys, cens)
)';

--------------------------------------------------------------------------------
--- Creaci de la taula Est_Pais_Any -------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE EST_PAIS_ANY(
  anys NUMBER,
  pais VARCHAR2(4 CHAR) CONSTRAINT FK_EST_PAIS_ANY REFERENCES ESTAT(codi),
  n_vot_fi NUMBER DEFAULT 0,
  CONSTRAINT PK_EST_PAIS_ANY PRIMARY KEY(anys, pais)
)';

--------------------------------------------------------------------------------  
--- Creaci de la taula Grup_Est------------------------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE TABLE GRUP_EST(
  codi VARCHAR2(20 CHAR) CONSTRAINT PK_GRUP_EST PRIMARY KEY,
  anys NUMBER,
  vot_mes_part VARCHAR2(50 CHAR) CONSTRAINT FK_1_GRUP_EST REFERENCES VOTACIO(titol),
  local_mes_vots VARCHAR2(50 CHAR),
  n_mai_vot NUMBER(5,2) DEFAULT 0,
  mes_participacio VARCHAR2(50 CHAR) CONSTRAINT FK_2_GRUP_EST REFERENCES VOTACIO(titol),
  mes_dif_opc VARCHAR2(50 CHAR) CONSTRAINT FK_3_GRUP_EST REFERENCES VOTACIO(titol),
  menys_dif_opc VARCHAR2(50 CHAR) CONSTRAINT FK_4_GRUP_EST REFERENCES VOTACIO(titol),
  Ciutada NUMBER,
  Pais VARCHAR2(4 CHAR),
  n_votacions NUMBER DEFAULT 0
)';

END;
/