--------------------------------------------------------------------------------
------------------| Procediments Emmagatzemats |--------------------------------
--------------------------------------------------------------------------------

BEGIN
--------------------------------------------------------------------------------
---1----- Procediment per donar d''alta una nova Votaci ------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Alta_Votacio 
(tit IN VARCHAR2, data_p IN DATE, data_o IN DATE, data_t IN DATE, presi IN VARCHAR2, 
 descrip IN VARCHAR2, pweb VARCHAR2, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(400); --- Parmetres d''entrada  
  excepcio EXCEPTION; --- Nom de l''excepci
  da DATE;

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := tit || '', '' || data_p || '', '' || data_o || '', '' || data_t || '', '' 
                            || presi || '', '' || descrip || '', '' || pweb;
  
  --- Comprovem si ttol s buit
  IF tit IS NULL THEN
    tex_error := ''El ttol de la Votaci no pot ser buit.'';
    RAISE excepcio;
  ELSIF presi IS NULL THEN
    tex_error := ''El President de la Votaci no pot ser buit.'';
    RAISE excepcio;
  ELSIF descrip IS NULL THEN
    tex_error := ''La descripci de la Votaci no pot ser buida.'';
    RAISE excepcio;
  END IF;
  --- Comprovem si la data de publicaci s buida
  IF data_p IS NULL THEN da := SYSDATE; ELSE da := data_p; END IF;
  
  --- Creaci d''una nova Votaci
  INSERT INTO VOTACIO(titol, data_publicacio, data_obertura, data_tancament,  president, descripcio, web)
  VALUES(tit, da, data_o, data_t, presi, descrip, pweb);
  
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Alta_Votacio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractament dels errors personalitzat
  WHEN excepcio THEN
    n_error := SQLCODE;
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Votacio'', entrades, RSP);
  --- Tractament de la resta d''errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11, 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Votacio'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---2----- Procediment per donar de Baixa una Votaci existent ------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Baixa_Votacio (tit IN VARCHAR2, 
RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(45); --- Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci  
  auxiliar VARCHAR2(40);
  num NUMBER;

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := ''-> '' || tit;
  
  --- Si no hi ha cap valor en ttol llancem una excepci
  IF tit IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com a ttol.'';
    RAISE excepcio;
  END IF;
  SELECT COUNT(titol) INTO num FROM VOTACIO WHERE titol = tit;
  --- Si el ttol no existeix llancem una excepci
  IF num < 1 THEN
    tex_error := ''No existeix cap Votaci amb el nom: ''||tit||''.'';
    RAISE excepcio;
  END IF;
  --- Esbrinem l''estat de la Votaci
  SELECT estat INTO auxiliar FROM VOTACIO WHERE titol = tit;
  --- S s Oberta o tancada
  IF auxiliar = ''Oberta'' OR auxiliar = ''Tancada'' THEN
    tex_error := ''No es pot Eliminar una Votaci Oberta o que ja ha estat Tancada.'';
    RAISE excepcio;
  END IF;
  
  --- Baixa d''una Votaci
  DELETE FROM VOTACIO WHERE titol = tit;
  
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Baixa_Votacio'', entrades, RSP);
  
--- Definici de l''excepci
EXCEPTION
  --- Tractament dels errors personalitzat
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Votacio'', entrades, RSP);  
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Votacio'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---3----- Procediment per modificar una Votaci existent -----------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Modificar_Votacio 
(tit IN VARCHAR2, data_p IN date, data_o IN DATE, data_t IN DATE, presi IN VARCHAR2, 
 descrip IN VARCHAR2, pweb VARCHAR2, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci  
  auxiliar VARCHAR2(40);
  num NUMBER;

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := tit || '', '' || data_p || '', '' || data_o || '', '' || data_t || '', '' 
                          || presi || '', '' || descrip || '', '' || pweb;
                            
  --- Si no hi ha cap valor en ttol llancem una excepci
  IF tit IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com a ttol.'';
    RAISE excepcio;
  END IF;
  --- Si el ttol no existeix llancem una excepci
  SELECT COUNT(titol) INTO num FROM VOTACIO WHERE titol = tit;
  --- Si el ttol no existeix llancem una excepci
  IF num < 1 THEN
    tex_error := ''No existeix cap Votaci amb el nom: ''||tit||''.'';
    RAISE excepcio;
  END IF;
  
  --- Esbrinem l''estat de la Votaci
  SELECT estat INTO auxiliar FROM VOTACIO WHERE titol = tit;
  IF auxiliar <> ''Edici'' THEN
    tex_error := ''La Votaci ''||tit||'' es troba en Estat: ''||auxiliar||''. No permet modificacions.'';
    RAISE excepcio;
  END IF;
  --- Si el parmetre d''entrada corresponent t alguna dada el modifiquem
  --- Data de publicaci
  IF data_p IS NOT NULL THEN
  UPDATE VOTACIO SET data_publicacio = data_p WHERE titol = tit;
  END IF;
  --- Data d''obertura
  IF data_o IS NOT NULL THEN
  UPDATE VOTACIO SET data_obertura = data_o WHERE titol = tit;
  END IF;
  --- Data de tancament
  IF data_t IS NOT NULL THEN
  UPDATE VOTACIO SET data_tancament = data_t WHERE titol = tit;
  END IF;
  --- Nom del president
  IF presi IS NOT NULL THEN
  UPDATE VOTACIO SET president = presi WHERE titol = tit;
  END IF;
  --- Descripci
  IF descrip IS NOT NULL THEN
  UPDATE VOTACIO SET descripcio = descrip WHERE titol = tit;
  END IF;
  --- Web
  IF pweb IS NOT NULL THEN
  UPDATE VOTACIO SET web = pweb WHERE titol = tit;
  END IF;
  
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Modificar_Votacio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractament dels errors personalitzat
  WHEN excepcio THEN
    n_error := SQLCODE;
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Modificar_Votacio'', entrades, RSP);
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Modificar_Votacio'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---4----- Procediment per modificar l''estat d''una votaci existent -------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Modificar_Estat_Votacio 
(tit IN VARCHAR2, est IN VARCHAR2, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(50); --- Parmetres d''entrada
  CURSOR C1 IS SELECT cens FROM ASSIGNACIO WHERE votacio = tit; --- Censos assignats
  excepcio EXCEPTION; --- Nom de l''excepci
  num NUMBER;
  da DATE;
  an NUMBER;  
  auxiliar VARCHAR2(60);

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := tit || '', '' || est;
  
  --- Comprovacions de seguretat
  --- Si no hi ha cap valor en ttol o Estat llancem una excepci
  IF tit IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com a titol.'';
    RAISE excepcio;
  END IF;
  --- Si no hi ha cap valor en ttol o Estat llancem una excepci
  IF est IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com Estat.'';
    RAISE excepcio;
  END IF;
  --- Comprovem si el ttol existeix
  SELECT COUNT(titol) INTO num FROM VOTACIO WHERE titol = tit;
  --- Si el ttol no existeix llancem una excepci
  IF num < 1 THEN
    tex_error := ''No existeix cap Votaci amb el nom: ''||tit||''.'';
    RAISE excepcio;
  END IF;
  --- Si la Votaci ja s tancada
  SELECT estat INTO auxiliar FROM VOTACIO WHERE titol = tit;
  IF auxiliar = ''Tancada'' THEN 
    tex_error := ''No es pot modificar l''''Estat quan la Votaci ja s tancada.'';
    RAISE excepcio; 
  END IF;
  --- Si l''estat introdut no s corecte llancem una excepci
  IF est NOT IN(''Edici'',''Pendent'',''Oberta'',''Tancada'') THEN
    tex_error := ''L''''Estat: ''||est||'' s incorrecte'';
    RAISE excepcio;
  END IF;
  --- No es permeten Votacions amb menys de dos Opcions ni sense tenir censos assignats
  IF est = ''Pendent'' OR est = ''Oberta'' THEN
    SELECT COUNT(codi) INTO num FROM OPCIO WHERE votacio = tit;
    IF num < 2 THEN 
      tex_error := ''No es permeten Votacions amb menys de dos Opcions.'';
      RAISE excepcio; 
    END IF;
    --- Assignaci de censos
    SELECT COUNT(cens) INTO num FROM ASSIGNACIO WHERE votacio = tit;
      IF num < 1 THEN 
        tex_error := ''No es permeten Votacions sense Censos assignats.'';
        RAISE excepcio; 
      END IF;
  END IF;
    
  --- Modifiquem l''estat de la Votaci  
  IF est = ''Edici'' OR est = ''Pendent'' OR est = ''Oberta'' THEN 
    --- Per Obrir una Votaci abans ha d''estar Pendent
    IF est = ''Oberta'' THEN
      SELECT estat INTO auxiliar FROM VOTACIO WHERE titol = tit;
      IF auxiliar <> ''Pendent'' THEN 
        tex_error := ''No es pot passar a un estat: ''||est||'', si abans era ''||auxiliar||''.'';
        RAISE excepcio; 
      END IF;      
    END IF;
    --- Actualitzem l''estat
    UPDATE VOTACIO SET estat = est WHERE titol = tit;
    --- Actualitzem el nmero definitiu de votants als censos assignats*****
     --- Obrim el cursor de censos
    OPEN C1;
    LOOP
      --- Fem un recorregut per tots ells
      FETCH C1 INTO auxiliar;      
      --- Sortim quan no hi hagin ms      
      EXIT WHEN C1%NOTFOUND;
      --- Actualitzem la dada
      --- Nmero fins ara
      UPDATE ASSIGNACIO 
      SET votants = (SELECT COUNT (ciutada) FROM INSCRIPCIO 
                     WHERE cens = auxiliar AND data_ins < (SELECT data_obertura 
                                                            FROM VOTACIO 
                                                            WHERE titol = tit)) 
      WHERE cens = auxiliar AND votacio = tit;  
    END LOOP;
    --- Tanquem el cursor
    CLOSE C1;
    
  END IF;
  --- Tanquem la Votaci
  IF est = ''Tancada'' THEN
      --- Comprovem que la Votaci est Oberta en aquests moments
      SELECT estat INTO auxiliar FROM VOTACIO WHERE titol = tit;
      IF auxiliar <> ''Oberta'' THEN 
        tex_error := ''No es pot passar a un estat: ''||est||'', si abans era ''||auxiliar||''.'';
        RAISE excepcio; 
      END IF;
      --- Actualitzem l''estat
      UPDATE VOTACIO SET estat = est WHERE titol = tit;
      
      --- Accions de tancament d''una Votaci
      --- Traiem l''any de la data de tancament de la Votaci
      SELECT data_tancament INTO da FROM VOTACIO WHERE titol = tit;
      SELECT EXTRACT(YEAR FROM da) INTO an FROM DUAL;
      --- Comprovem si la votaci s d''mbit Global
      SELECT COUNT(cens) INTO num FROM ASSIGNACIO WHERE votacio = tit AND cens = ''Europa'';
      IF num = 1 THEN
        --- Augmentem els registres en una unitat per a tots el paissos de la taula
        UPDATE EST_PAIS_ANY 
          SET n_vot_fi = n_vot_fi + 1
        WHERE anys = an AND pais IN (SELECT codi FROM ESTAT WHERE ESTAT.codi = EST_PAIS_ANY.PAIS);
      --- Si no t assignat el cens Global
      ELSE
        --- Augmentem els registres en una unitat per a tots el paissos dels Censos assignats
        UPDATE EST_PAIS_ANY 
          SET n_vot_fi = n_vot_fi + 1
        WHERE anys = an AND pais IN (SELECT DISTINCT cee.estat FROM CENS cen, ASSIGNACIO asi, CENS_ESTAT cee
                                      WHERE asi.votacio = tit AND cen.nom = asi.cens AND cen.nom = cee.nom
                                      UNION
                                      SELECT DISTINCT cer.estat FROM CENS cen, ASSIGNACIO asi, CENS_REGIO cer
                                      WHERE asi.votacio = tit AND cen.nom = asi.cens AND cen.nom = cer.nom
                                      UNION
                                      SELECT DISTINCT cel.estat FROM CENS cen, ASSIGNACIO asi, CENS_LOCAL cel
                                      WHERE asi.votacio = tit AND cen.nom = asi.cens AND cen.nom = cel.nom);
      END IF;
      --- Actualitzem el registre v_mig_part de la taula EST_CENS_ANY****
      --- Obrim el cursor de censos
      OPEN C1;
      LOOP
        --- Fem un recorregut per tots ells
        FETCH C1 INTO auxiliar;
      
        --- Els repassem tots
        EXIT WHEN C1%NOTFOUND;
        --- Actualitzem les dades
        UPDATE EST_CENS_ANY
        --- Valor mig de participaci de les Votacions tancades aquest mateiex any
        SET v_mig_part =(SELECT AVG(asi.participacio) FROM VOTACIO vo, ASSIGNACIO asi 
                         WHERE (SELECT EXTRACT(YEAR FROM vo.data_tancament) FROM DUAL) = an  
                                AND vo.titol = asi.votacio AND asi.cens = auxiliar)
        WHERE anys = an AND cens = auxiliar;
      END LOOP;
      --- Tanquem el cursor
      CLOSE C1;      
            
  END IF;
    
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Modificar_Estat_Votacio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractament dels errors personalitzat
  WHEN excepcio THEN
    n_error := SQLCODE;
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Modificar_Estat_Votacio'', entrades, RSP);
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    ---RSP := ''ERROR: '' || tex_error;
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Modificar_Estat_Votacio'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---5----- Procediment per donar d''alta un nou Ciutad --------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Alta_Ciutada 
(data_n IN DATE, loc IN VARCHAR2, reg IN VARCHAR2, est IN VARCHAR2, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Recull de Parmetres d''entrada
  excepcio EXCEPTION;
  num NUMBER; --- Variable contador
  idc NUMBER; --- Recullir el codi del Ciutad
BEGIN
  --- Parmetres d''entrada, introducci de dades
  entrades := data_n || '', '' || loc || '', '' || reg || '', '' || est;
  
  --- Comprobem si el ciutad t ms de 18 anys
  IF data_n IS NULL THEN
    tex_error := ''No s''''admeten valors buits com data de naixement.'';
    RAISE excepcio;
  END IF;
  -- Clcul de l''edat
  SELECT trunc((months_between(SYSDATE, data_n))/12) INTO num from dual;
  IF num < 18 THEN
    tex_error := ''No es poden admetre Ciutadans menors de 18 anys.'';
    RAISE excepcio;
  END IF;
  
  --- Comprovem el lloc de residncia del Ciutad
  --- Comprovem si la Localitat figura a la BD
  SELECT COUNT(codi) INTO num FROM LOCALITAT WHERE codi = loc AND regio = reg AND estat = est;
  IF loc IS NULL OR reg IS NULL OR est IS NULL THEN
    tex_error := ''No s''''admeten valors buits com Localitat.'';
    RAISE excepcio;
  ELSIF num = 0 THEN
    tex_error := ''La Localitat: ''||est||'', ''||reg||'', ''||loc||'' no existeix al sistema.'';
    RAISE excepcio;
  END IF;
  
  --- Generaci del codi del nou Ciutad
  LOOP
    -- Generaci d''un nmero aleatori positiu suficientment gran
    SELECT (1 + ABS(MOD(dbms_random.random, 9000000000))) INTO idc FROM dual;
    --- Si el codi existeix continuem provant
    SELECT COUNT(codi) INTO num FROM CIUTADA WHERE codi = idc;
    --- Si el codi no existeix el memoritzem i sortim del bucle
    EXIT WHEN (num < 1);
  END LOOP;
  
  --- Creaci d''un nou Ciutad
  INSERT INTO CIUTADA VALUES(idc, data_n, loc, reg, est, 0);
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Incorporem a les entrades el codi del ciutad per ms claredad
  entrades := idc || '', '' || entrades;
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Alta_Ciutada'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractament dels menors d''edat
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Ciutada'', entrades, RSP);
  --- Tractament de la resta d''errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Ciutada'', entrades, RSP);  
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---6----- Procediment per donar de baixa un Ciutad existent -------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Baixa_Ciutada (cod IN NUMBER, 
RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Per Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci
  num NUMBER;

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := ''-> '' || cod;
  
  --- Si no hi ha cap valor en cod llancem una excepci
  IF cod IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com a codi.'';
    RAISE excepcio;
  END IF;
  
  --- Esbinem si el codi existeix
  SELECT COUNT(codi) INTO num FROM CIUTADA WHERE codi = cod;
  --- Si el codi no existeix llancem una excepci
  IF num < 1 THEN
    tex_error := ''No existeix cap Ciutad amb el codi: ''||cod||''.'';
    RAISE excepcio;
  END IF;
  
  --- Esborrem el Ciutad
  DELETE FROM CIUTADA WHERE codi = cod;
     
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Baixa_Ciutada'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractaments dels errors personalitzats
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Ciutada'', entrades, RSP);  
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Ciutada'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---7----- Procediment per modificar un Ciutad existent ------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Modificar_Ciutada 
(cod IN NUMBER, data_n IN DATE, loc IN VARCHAR2, reg IN VARCHAR2, est IN VARCHAR2, 
 RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Per Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci
  num NUMBER;
  auxiliar VARCHAR2(40);
  auxiliar1 VARCHAR2(40);
  auxiliar2 VARCHAR2(40);

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := cod || '', '' || data_n || '', '' || loc || '', '' || reg || '', '' || est;
  --- Si no hi ha cap valor en cod llancem una excepci
  IF cod IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com a codi.'';
    RAISE excepcio;
  END IF;
  --- Si el codi no existeix llancem una excepci
  SELECT COUNT(codi) INTO num FROM CIUTADA WHERE codi = cod;
  --- Si el codi no existeix llancem una excepci
  IF num < 1 THEN
    tex_error := ''No existeix cap Ciutad amb el codi: ''||cod||''.'';
    RAISE excepcio;
  END IF;  
  
  --- Modifiquem el Ciutad
  --- Si el parmetre d''entrada corresponent t alguna dada el modifiquem sino no caldr
  --- Comprobem si el ciutad t ms de 18 anys
  -- Clcul de l''edat
  SELECT trunc((months_between(SYSDATE, data_n))/12) INTO num from dual;
  IF num < 18 THEN
    tex_error := ''No es pot admetre un Ciutad de: ''||num||'', s menor de 18 anys.'';
    RAISE excepcio;
  END IF;
  --- Modificaci de la data de neixement
  IF data_n IS NOT NULL THEN
  UPDATE CIUTADA SET edat = data_n WHERE codi = cod;
  END IF;  
    
  --- Busquem si la Localitat s diferent a l''existent
  SELECT localitat, regio, estat INTO auxiliar, auxiliar1, auxiliar2 FROM CIUTADA WHERE codi = cod;  
  IF auxiliar <> loc AND loc IS NOT NULL THEN    
    --- Si la Localitat no existeix llancem una excepci
    SELECT COUNT(codi) INTO num FROM LOCALITAT WHERE codi = loc AND regio = reg AND estat = est;
    IF num < 1 THEN
      tex_error := ''No existeix cap Localitat amb el codi: ''||loc||''.'';
      RAISE excepcio;
    END IF;
    UPDATE CIUTADA SET localitat = loc, regio = reg, estat = est 
    WHERE codi = cod;
    --- Esborrem la Inscripci al cens Local d''aquest Ciutad
    DELETE FROM INSCRIPCIO WHERE ciutada = cod AND cens = (''Loc_'' || auxiliar);
    --- Actualitzem el registre de nmero de votants totals a Cens
    UPDATE CENS SET votants = votants - 1 WHERE nom = (''Loc_'' || auxiliar);
    --- Si ha canviat de Regi esborrem la seva Inscripci al Cens Regional
    IF auxiliar1 <> reg AND reg IS NOT NULL THEN
      DELETE FROM INSCRIPCIO WHERE ciutada = cod AND cens = (''Reg_'' || auxiliar1);
      --- Actualitzem el registre de nmero de votants totals a Cens
      UPDATE CENS SET votants = votants - 1 WHERE nom = (''Reg_'' || auxiliar1);
      --- Si ha canviat d''Estat esborrem la seva Inscripci al Cens Estatal
      IF auxiliar2 <> est AND est IS NOT NULL THEN
        DELETE FROM INSCRIPCIO WHERE ciutada = cod AND cens = (''Est_'' || auxiliar2);
        --- Actualitzem el registre de nmero de votants totals a Cens
        UPDATE CENS SET votants = votants - 1 WHERE nom = (''Est_'' || auxiliar2);
      END IF;
    END IF;
  END IF;
  
  --- Si no hi ha cap error posem OK al RSP (parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Modificar_Ciutada'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractament dels errors personalitzat
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Modificar_Ciutada'', entrades, RSP);
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Modificar_Ciutada'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---8----- Procediment per donar d''alta un nou Cens -----------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Alta_Cens 
(ambit IN VARCHAR2, est IN VARCHAR2, reg IN VARCHAR2, loc IN VARCHAR2, 
RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Recull de Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci
  num NUMBER; --- Variable contador  
BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := ambit || '', '' || est || '', '' || reg || '', '' || loc;
  
  --- Comprobem l''mbit del Cens: ''Global'', ''Estatal'', ''Regional'', ''Local''
  IF ambit = ''Global'' THEN
    SELECT COUNT(ambit) INTO num FROM CENS WHERE ambit = ''Global''; 
    IF num = 0 THEN
      INSERT INTO CENS VALUES (''Europa'', ambit, 0);
    ELSE
      tex_error := ''Noms pot haver un Cens Global al sistema.'';
      RAISE excepcio;
    END IF;
  ELSIF ambit = ''Estatal'' THEN
    --- Comprovem si l''Estat figura a la BD
    SELECT COUNT(codi) INTO num FROM ESTAT WHERE codi = est;
    IF est IS NULL THEN
      tex_error := ''No s''''admeten valors buits com Estat si l''''mbit s ''||ambit||''.'';
      RAISE excepcio;
    ELSIF num = 0 THEN
      tex_error := ''L''''Estat: ''||est||'' no existeix al sistema.'';
      RAISE excepcio;
    END IF;    
    --- Insertem una entrada a la taula Cens
    INSERT INTO CENS VALUES (''Est_'' || est, ambit, 0);
    --- Insertem una entrada a la taula del seu mbit
    INSERT INTO CENS_ESTAT VALUES (''Est_'' || est, est);
  ELSIF ambit = ''Regional'' THEN
    --- Comprovem si la Regi figura a la BD
    SELECT COUNT(codi) INTO num FROM REGIO WHERE codi = reg;
    IF reg IS NULL OR est IS NULL THEN
      tex_error := ''No s''''admeten valors buits com Regi si l''''mbit s ''||ambit||''.'';
      RAISE excepcio;
    ELSIF num = 0 THEN
      tex_error := ''La Regi: ''||reg||'' no existeix al sistema.'';
      RAISE excepcio;
    END IF; 
    --- Insertem una entrada a la taula Cens
    INSERT INTO CENS VALUES (''Reg_'' || reg, ambit, 0);
    --- Insertem una entrada a la taula del seu mbit
    INSERT INTO CENS_REGIO VALUES (''Reg_'' || reg, est, reg);
  ELSIF ambit = ''Local'' THEN
    --- Comprovem si la Localitat figura a la BD
    SELECT COUNT(codi) INTO num FROM LOCALITAT WHERE codi = loc;
    IF loc IS NULL OR reg IS NULL OR est IS NULL THEN
      tex_error := ''No s''''admeten valors buits com Localitat si l''''mbit s ''||ambit||''.'';
      RAISE excepcio;
    ELSIF num = 0 THEN
      tex_error := ''La Localitat: ''||est||'' no existeix al sistema.'';
      RAISE excepcio;
    END IF; 
    --- Insertem una entrada a la taula Cens
    INSERT INTO CENS VALUES (''Loc_'' || loc, ambit, 0);
    --- Insertem una entrada a la taula del seu mbit
    INSERT INTO CENS_LOCAL VALUES (''Loc_'' || loc, est, reg, loc);
  ELSIF ambit IS NULL THEN
    tex_error := ''El parmetre mbit no permet valors buits.'';
    RAISE excepcio;
  ELSE
    tex_error := ''El parmetre mbit no admet el valor: '' || ambit ||''.'';
    RAISE excepcio;
  END IF;
   
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Alta_Cens'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractament dels errors personalitzats
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Cens'', entrades, RSP);  
  --- Tractament de la resta d''errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Cens'', entrades, RSP);  
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---9----- Procediment per donar de baixa d''un Cens existent --------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Baixa_Cens (cen IN VARCHAR2, 
RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Per Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci
  num NUMBER;
  
BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := ''-> '' || cen;
  
  --- Si no hi ha cap valor en cen llancem una excepci
  IF cen IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com a nom.'';
    RAISE excepcio;
  END IF;
  --- Si el nom no existeix llancem una excepci
  SELECT COUNT(nom) INTO num FROM CENS WHERE nom = cen;
  --- Si el nom no existeix llancem una excepci
  IF num < 1 THEN
    tex_error := ''No existeix cap Cens amb el nom: ''||cen||''.'';
    RAISE excepcio;
  END IF;
  --- Si el Cens ha estat assignat a una Votaci llancem una excepci
  SELECT COUNT(cens) INTO num FROM ASSIGNACIO WHERE cens = cen;
  --- Si el nom existeix llancem una excepci
  IF num >= 1 THEN
    tex_error := ''No es pot esborrar el Cens amb nom: ''||cen||'' est assignat a ''||num||'' Votacions.'';
    RAISE excepcio;
  END IF;
  
  --- Esborrem el Cens
  DELETE FROM CENS WHERE nom = cen;
     
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Baixa_Cens'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractaments dels valors nuls
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Cens'', entrades, RSP);  
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Cens'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---10----- Procediment per donar d''alta una nova Opci -------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Alta_Opcio (vot IN VARCHAR2, 
des IN VARCHAR2, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Recull de Parmetres d''entrada 
  excepcio EXCEPTION; --- Nom de l''excepci
  auxiliar VARCHAR2(40);
  num NUMBER;

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := vot || '', '' || des;
  
  --- Esbrinem si la Votaci existeix a la BD
  SELECT COUNT(titol) INTO num FROM VOTACIO WHERE titol = vot;
  IF vot IS NULL THEN
    tex_error := ''La Votaci no permet valors buits.'';
    RAISE excepcio;
  ELSIF num = 0 THEN
    tex_error := ''La Votaci: ''||vot||'', no existeix al sistema.'';
    RAISE excepcio;
  END IF;
  
  --- Esbrinem si la descripci cont text
  IF des IS NULL THEN
    tex_error := ''La descripci no permet valors buits.'';
    RAISE excepcio;
  END IF;
  
  --- Esbrinem l''estat de la Votaci
  SELECT estat INTO auxiliar FROM VOTACIO WHERE titol = vot;
  --- Si la votaci ha deixat l''estat d''Edici o Pendent no podr eser modificada
  IF auxiliar = ''Edici'' THEN
    ---- Insertem una entrada a la taula Opcio
    INSERT INTO OPCIO (votacio, descripcio) VALUES (vot, des);
  ELSE
    tex_error := ''La Votaci es troba en Estat: ''||auxiliar||''. No permet modificacions.'';
    RAISE excepcio;
  END IF;
  
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Alta_Opcio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractament de l''estat de la Votaci
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Opcio'', entrades, RSP);
  --- Tractament de la resta d''errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Opcio'', entrades, RSP);  
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---11----- Procediment per donar de baixa una Opci existent -------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Baixa_Opcio (cod IN NUMBER, 
RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Per Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci
  num NUMBER;
  auxiliar VARCHAR2(40);

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := ''-> '' || cod;
  
  --- Si no hi ha cap valor en cod llancem una excepci
  IF cod IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com a codi.'';
    RAISE excepcio;
  END IF;
  --- Si el codi no existeix llancem una excepci
  SELECT COUNT(codi) INTO num FROM OPCIO WHERE codi = cod;
  --- Si el codi no existeix llancem una excepci
  IF num < 1 THEN
    tex_error := ''No existeix cap Opci amb el codi: ''||cod||''.'';
    RAISE excepcio;
  END IF;
  
  --- Esbrinem l''estat de la Votaci
  SELECT estat INTO auxiliar FROM VOTACIO v, OPCIO o WHERE o.codi = cod AND o.votacio = v.titol;
  --- Si la votaci ha deixat l''estat d''Edici o Pendent no podr eser modificada
  IF auxiliar = ''Edici'' THEN
    --- Esborrem l''Opci
    DELETE FROM OPCIO WHERE codi = cod;
  ELSE
    tex_error := ''La Votaci es troba en Estat: ''||auxiliar||''. No permet modificacions.'';
    RAISE excepcio;
  END IF;
     
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Baixa_Opcio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractaments dels errors personalitzats
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Opcio'', entrades, RSP);  
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Opcio'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---12----- Procediment per modificar una Opci existent ------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Modificar_Opcio (cod IN NUMBER, 
dcr IN VARCHAR2, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Per Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci 
  auxiliar VARCHAR2(40);
  num NUMBER;

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := cod || '', '' || dcr;
  
  --- Si no hi ha cap valor en cod llancem una excepci
  IF cod IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com a codi.'';
    RAISE excepcio;
  END IF;
  --- Si no hi ha cap valor en dcr llancem una excepci
  IF dcr IS NULL THEN
    tex_error := ''No es pot acceptar un valor buit com a descripci.'';
    RAISE excepcio;
  END IF;
  --- Si el codi no existeix llancem una excepci
  SELECT COUNT(codi) INTO num FROM OPCIO WHERE codi = cod;
  --- Si el codi no existeix llancem una excepci
  IF num < 1 THEN
    tex_error := ''No existeix cap Opci amb el codi: ''||cod||''.'';
    RAISE excepcio;
  END IF;
  --- Esbrinem l''estat de la Votaci
  SELECT estat INTO auxiliar FROM VOTACIO v, OPCIO o WHERE o.codi = cod AND o.votacio = v.titol;
  --- Si la votaci ha deixat l''estat d''Edici o Pendent no podr eser modificada
  IF auxiliar = ''Edici'' THEN
    --- Editem l''Opci
    UPDATE OPCIO SET descripcio = dcr WHERE codi = cod;
  ELSE
    tex_error := ''La Votaci es troba en Estat: ''||auxiliar||''. No permet modificacions.'';
    RAISE excepcio;
  END IF;
     
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Modificar_Opcio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractament dels errors personalitzat
  WHEN excepcio THEN
    n_error := SQLCODE;
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Modificar_Opcio'', entrades, RSP);
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Modificar_Opcio'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---13----- Procediment per Assignar un nou Cens a una Votaci ------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Alta_Assignacio(cen IN VARCHAR2, 
vot IN VARCHAR2, da IN DATE, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Recull de Parmetres d''entrada 
  excepcio EXCEPTION;
  auxiliar VARCHAR2(40);
  daf DATE;
  an NUMBER;
  num NUMBER;

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := cen || '', '' || vot || '', '' || da;
  
 --- Valors buits
  IF cen IS NULL THEN
    tex_error := ''No s''''ha escollit cap Cens.'';
    RAISE excepcio;
  END IF;
  IF vot IS NULL THEN
    tex_error := ''No s''''ha escollit cap Votaci.'';
    RAISE excepcio;
  END IF;
  IF da IS NULL THEN daf := SYSDATE; ELSE daf := da; END IF;
  --- Valors inexistents
  SELECT COUNT(nom) INTO num FROM CENS WHERE nom = cen;
  IF num = 0 THEN
    tex_error := ''El Cens: ''||cen||'' no existeix al sistema.'';
    RAISE excepcio;
  END IF;
  SELECT COUNT(titol) INTO num FROM VOTACIO WHERE titol = vot;
  IF num = 0 THEN
    tex_error := ''La Votaci: ''||vot||'' no existeix al sistema.'';
    RAISE excepcio;
  END IF;
  --- Assignacions a votacions no editables
  SELECT estat INTO auxiliar FROM VOTACIO WHERE titol = vot;
  IF auxiliar = ''Edici'' THEN
    --- Insertem una entrada a la taula Assignaci
    INSERT INTO ASSIGNACIO(cens, votacio, data_assig) VALUES (cen, vot, daf);    
  ELSE
    tex_error := ''No es poden assignar censos a votacions ja presentades.'';
    RAISE excepcio;
  END IF;
  
  --- Traiem l''any de la data
  SELECT EXTRACT(YEAR FROM daf) INTO an FROM DUAL;
  --- Comprovem si hi ha un registre amb aquestes dades
  SELECT COUNT(cens) INTO num FROM EST_CENS_ANY WHERE anys = an AND cens = cen;
  --- Si existeix
  IF num = 1 THEN
    --- Augmentem el nmero de Censos assignats en una unitat
    UPDATE EST_CENS_ANY SET n_vot_asoc = n_vot_asoc + 1 WHERE  anys = an AND cens = cen;
  --- Si no existeix
  ELSE
    --- Insertem un nou registre amb valor inicial 1
    INSERT INTO EST_CENS_ANY (anys, cens, n_vot_asoc) VALUES (an, cen, 1);
  END IF;
  
  --- Actualitzem el registre de nmero de votants totals a Votaci i al Cens en aquesta assignaci
  --- Nmero en aquests moments
  SELECT votants INTO num FROM CENS WHERE nom = cen;
  --- Actualitzaci
  UPDATE VOTACIO SET total_ciutadans = total_ciutadans + num WHERE titol = vot;
  --- Nmero fins ara
  UPDATE ASSIGNACIO SET votants = (SELECT votants FROM CENS WHERE nom = cen) 
  WHERE cens = cen AND votacio = vot;
  
  --- Afegim un registre a EST_PAIS_ANY iniciat a 0
  SELECT data_tancament INTO daf FROM VOTACIO WHERE titol = vot;
  --- Traiem l''any de la data
  SELECT EXTRACT(YEAR FROM daf) INTO an FROM DUAL;
  --- Insertem una lnea nova per aquest any de tots els paissos si no hi son
  INSERT INTO EST_PAIS_ANY (anys, pais, n_vot_fi) 
  SELECT an, e.codi, 0 FROM ESTAT e WHERE e.codi NOT IN (SELECT pais FROM EST_PAIS_ANY WHERE anys = an);
   
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Alta_Assignacio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Errors personalitzats
  WHEN excepcio THEN
    n_error := SQLCODE;
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Assignacio'', entrades, RSP);
  --- Tractament de la resta d''errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Assignacio'', entrades, RSP);  
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---14----- Procediment per donar de baixa una Assignaci existent --------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Baixa_Assignacio(cen IN VARCHAR2, 
vot IN VARCHAR2, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Per Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci
  auxiliar VARCHAR2(50);
  da DATE;
  num NUMBER;
  an NUMBER;
  
BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := cen || '', '' || vot;
  
  --- Si no hi ha cap valor en cens o votacio llancem una excepci
  IF cen IS NULL THEN
    tex_error := ''No es pot acceptar un valor Buit com a Cens.'';
    RAISE excepcio;
  END IF;
  IF vot IS NULL THEN
    tex_error := ''No es pot acceptar un valor Buit com a Votaci.'';
    RAISE excepcio;
  END IF;
  --- Si l''Assignaci no existeix llancem una excepci
  SELECT COUNT(cens) INTO num FROM ASSIGNACIO WHERE cens = cen AND votacio = vot;
  IF num < 1 THEN
    tex_error := ''No existeix cap Assignaci del Cens: ''||cen||'' a la Votaci: ''||vot||''.'';
    RAISE excepcio;
  END IF;
  
  --- Traiem l''any de la data en l''Assignaci que esborrem
  SELECT data_assig INTO da FROM ASSIGNACIO WHERE cens = cen AND votacio = vot;
  SELECT EXTRACT(YEAR FROM da) INTO an FROM DUAL;
  
  --- Noms es podran modificar Votacions en fase d''Edici
  SELECT estat INTO auxiliar FROM VOTACIO WHERE titol = vot;
  IF auxiliar = ''Edici'' THEN
    --- Esborrem l''Assignaci
    DELETE FROM ASSIGNACIO WHERE cens = cen AND votacio = vot;
  ELSE
    tex_error := ''No es pot treure assignacions de votacions ja publicades.'';
    RAISE excepcio;
  END IF;
  
  --- Actualitzem les estadstiques
  
  --- Comprovem si hi ha un registre amb aquestes dades
  SELECT COUNT(cens) INTO num FROM EST_CENS_ANY WHERE anys = an AND cens = cen;
  --- Si existeix
  IF num = 1 THEN
    --- Disminum el nmero de Censos assignats en una unitat
    UPDATE EST_CENS_ANY SET n_vot_asoc = n_vot_asoc - 1 WHERE  anys = an AND cens = cen;
  END IF;
  
  --- Actualitzem el registre de nmero de votants totals a Votaci
  --- Nmero de votants en aquests moments
  SELECT SUM(votants) INTO num FROM ASSIGNACIO WHERE votacio = vot;
  --- Actualitzaci
  IF num IS NULL THEN
    UPDATE VOTACIO SET total_ciutadans = 0 
    WHERE titol = vot;
  ELSE
    UPDATE VOTACIO SET total_ciutadans = num 
    WHERE titol = vot;
  END IF;
  
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Baixa_Assignacio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Assignaci fora de termini
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Assignacio'', entrades, RSP);  
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Assignacio'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---15----- Procediment per inscriure un Ciutad als seus Censos ----------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Alta_Inscripcio (cod IN NUMBER, 
da IN DATE, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Recull de Parmetres d''entrada  
  auxiliar VARCHAR(40);
  excepcio EXCEPTION;
  num NUMBER;
  d DATE;

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := cod || '', '' || da;
  
  --- Comprovem si el Ciutad existeix
  SELECT COUNT(codi) INTO num FROM CIUTADA WHERE codi = cod;
  IF cod IS NULL THEN 
    tex_error := ''No s''''admeten valors buits com Ciutad.'';
    RAISE excepcio; 
  END IF;
  IF num = 0 THEN 
    tex_error := ''El Ciutad amb el codi: ''||cod||'', no existeix al sistema.'';
    RAISE excepcio; 
  END IF;
  --- Comprovem si la data s buida
  IF da IS NULL THEN d := SYSDATE; ELSE d := da; END IF;
  
  --- Comprovem si els Censos als qu pertany Ciutad estan donats d''alta
  SELECT COUNT(nom) INTO num FROM CENS WHERE nom = ''Europa''; 
  IF num = 0 THEN
    auxiliar := ''Global'';
    tex_error := ''El Cens ''||auxiliar||'' del ciutad: ''||cod||'', no existeix.'';
    RAISE excepcio;
  END IF;
  SELECT COUNT(estat) INTO num FROM CENS_ESTAT WHERE estat IN (SELECT estat FROM CIUTADA WHERE codi = cod);
  IF num = 0 THEN
    auxiliar := ''Estatal'';
    tex_error := ''El Cens ''||auxiliar||'' del ciutad: ''||cod||'', no existeix.'';
    RAISE excepcio;
  END IF;
  SELECT COUNT(regio) INTO num FROM CENS_REGIO WHERE regio IN (SELECT regio FROM CIUTADA WHERE codi = cod);
  IF num = 0 THEN
    auxiliar := ''Regional'';
    tex_error := ''El Cens ''||auxiliar||'' del ciutad: ''||cod||'', no existeix.'';
    RAISE excepcio;
  END IF;
  SELECT COUNT(localitat) INTO num FROM CENS_LOCAL WHERE localitat IN (SELECT localitat FROM CIUTADA WHERE codi = cod);
  IF num = 0 THEN
    auxiliar := ''Local'';
    tex_error := ''El Cens ''||auxiliar||'' del ciutad: ''||cod||'', no existeix.'';
    RAISE excepcio;
  END IF;
  
  --- Comprovem si el Ciutad ja est inscrit
  SELECT COUNT(ciutada) INTO num FROM INSCRIPCIO WHERE ciutada = cod AND cens = ''Europa'';
  IF num = 0 THEN
    --- Inscrivim el Ciutad al Cens Global
    INSERT INTO INSCRIPCIO(ciutada, cens, data_ins) VALUES (cod, ''Europa'', d);  
    --- Actualitzem el registre de nmero de votants totals a Cens
    UPDATE CENS SET votants = votants + 1 WHERE nom = ''Europa'';
  END IF;
  
  --- Inscrivim al ciutad al seu Cens Estatal
  SELECT nom INTO auxiliar FROM CENS_ESTAT WHERE estat IN (SELECT estat FROM CIUTADA WHERE codi = cod);
  --- Comprovem si el Ciutad ja est inscrit
  SELECT COUNT(ciutada) INTO num FROM INSCRIPCIO WHERE ciutada = cod AND cens = auxiliar;
  IF num = 0 THEN
    INSERT INTO INSCRIPCIO(ciutada, cens, data_ins) VALUES (cod, auxiliar, d);  
    --- Actualitzem el registre de nmero de votants totals a Cens
    UPDATE CENS SET votants = votants + 1 WHERE nom = auxiliar;
  END IF;
  
  --- Inscrivim al ciutad al seu Cens Regional
  SELECT nom INTO auxiliar FROM CENS_REGIO WHERE regio IN (SELECT regio FROM CIUTADA WHERE codi = cod);
  --- Comprovem si el Ciutad ja est inscrit
  SELECT COUNT(ciutada) INTO num FROM INSCRIPCIO WHERE ciutada = cod AND cens = auxiliar;
  IF num = 0 THEN
    INSERT INTO INSCRIPCIO(ciutada, cens, data_ins) VALUES (cod, auxiliar, d);  
    --- Actualitzem el registre de nmero de votants totals a Cens
    UPDATE CENS SET votants = votants + 1 WHERE nom = auxiliar; 
  END IF;
    
  --- Inscrivim al ciutad al seu Cens Local
  SELECT nom INTO auxiliar FROM CENS_LOCAL WHERE localitat IN (SELECT localitat FROM CIUTADA WHERE codi = cod);
  --- Comprovem si el Ciutad ja est inscrit
  SELECT COUNT(ciutada) INTO num FROM INSCRIPCIO WHERE ciutada = cod AND cens = auxiliar;
  IF num = 0 THEN
    INSERT INTO INSCRIPCIO(ciutada, cens, data_ins) VALUES (cod, auxiliar, d);  
    --- Actualitzem el registre de nmero de votants totals a Cens
    UPDATE CENS SET votants = votants + 1 WHERE nom = auxiliar; 
  END IF;
      
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Alta_Inscripcio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractement dels errors personalitzats
  WHEN excepcio THEN
    n_error := SQLCODE;    
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Inscripcio'', entrades, RSP);  
  --- Tractament de la resta d''errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Alta_Inscripcio'', entrades, RSP);  
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---16----- Procediment per donar de baixa una Inscripci existent --------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Baixa_Inscripcio (ciu IN NUMBER, 
cen IN VARCHAR2, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Per Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci
  num NUMBER;

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := ciu || '', '' || cen;
  
  --- Si no hi ha cap valor en cens o ciutad llancem una excepci
  IF ciu IS NULL THEN
    tex_error := ''No es pot acceptar un valor Buit com a Ciutad.'';
    RAISE excepcio;
  END IF;
  IF cen IS NULL THEN
    tex_error := ''No es pot acceptar un valor Buit com a Cens.'';
    RAISE excepcio;
  END IF;
  --- Si la Inscripci no existeix llancem una excepci
  SELECT COUNT(cens) INTO num FROM INSCRIPCIO 
  WHERE cens = cen AND ciutada = ciu;
  IF num < 1 THEN
    tex_error := ''No existeix cap Inscripci d''''un Ciutad: ''||ciu||'' al Cens: ''||cen||''.'';
    RAISE excepcio;
  END IF;
  
  --- Esborrem l''Assignaci
  DELETE FROM INSCRIPCIO WHERE cens = cen AND ciutada = ciu;
  --- Actualitzem el registre de nmero de votants totals a Cens
  UPDATE CENS SET votants = votants - 1 WHERE nom = cen;
     
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Baixa_Inscripcio'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractaments dels valors nuls
  WHEN excepcio THEN
    n_error := SQLCODE;
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Inscripcio'', entrades, RSP);
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Baixa_Inscripcio'', entrades, RSP);
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---17----- Procediment per permetre Votar a un Ciutad -------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE Votar 
(vot IN VARCHAR2, cod IN NUMBER, opc IN NUMBER, da IN DATE, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Recull de Parmetres d''entrada  
  auxiliar VARCHAR(40);
  CURSOR C1 IS SELECT codi FROM OPCIO WHERE votacio = vot;
  excepcio EXCEPTION;
  da1 DATE;
  num NUMBER;
  num2 NUMBER;  

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := vot || '', '' || cod || '', '' || opc || '', '' || da;
  
  --- Comprovem els valors buits
  IF vot IS NULL THEN
    tex_error := ''La Votaci no admet valors buits.'';
    RAISE excepcio;
  END IF;
  IF cod IS NULL THEN
    tex_error := ''El codi del Ciutad no admet valors buits.'';
    RAISE excepcio;
  END IF;
  IF opc IS NULL THEN
    tex_error := ''L''''Opci de Vot no permet valors buits.'';
    RAISE excepcio;
  END IF;
  --- Incorporem la data del sistema com a data de Vot si no la personalitzem
  IF da IS NULL THEN da1 := SYSDATE; ELSE da1 := da; END IF;
  
  --- Comprovem que la Votaci existeix****
  SELECT COUNT(titol) INTO num FROM VOTACIO WHERE titol = vot;
  IF num = 0 THEN
    tex_error := ''La Votaci: ''||vot||'' no existeix.'';
    RAISE excepcio;
  END IF;
  
  --- Comprovem si la data de votaci s correcta
  SELECT COUNT(titol) INTO num 
  FROM VOTACIO 
  WHERE titol = vot AND da1 BETWEEN data_obertura AND data_tancament;
  IF num = 0 THEN
    tex_error := ''La data: ''||da1||'' s fora del termini de Votaci.'';
    RAISE excepcio;
  END IF;
  --- Comprovem si la Votaci s oberta
  SELECT COUNT(titol) INTO num 
  FROM VOTACIO 
  WHERE titol = vot AND estat = ''Oberta'';
  IF num = 0 THEN
    tex_error := ''La Votaci: ''||vot||'' no s Oberta.'';
    RAISE excepcio;
  END IF;
  
  --- Comprovem si l''Opci escollida s vlida
  SELECT COUNT(codi) INTO num 
  FROM OPCIO 
  WHERE codi = opc AND votacio = vot;
  IF num = 0 THEN
    tex_error := ''L''''Opci: ''||opc||'' no pertany a aquesta Votaci.'';
    RAISE excepcio;
  END IF;
  
  --- Comprovem si el Ciutad t dret a Vot
  SELECT COUNT(ins.ciutada) INTO num FROM INSCRIPCIO ins, ASSIGNACIO asi 
  WHERE asi.votacio = vot AND asi.cens = ins.cens AND ins.ciutada = cod; 
  IF num = 0 THEN
    tex_error := ''El Ciutad: ''||cod||'' no est inscrit als censos.'';
    RAISE excepcio;
  END IF;
  --- Comprovem si la Inscripci del Ciutad s dins el termini
  SELECT COUNT(ins.ciutada) INTO num FROM INSCRIPCIO ins, VOTACIO vo, ASSIGNACIO asi 
  WHERE asi.votacio = vot AND asi.cens = ins.cens  AND ins.ciutada = cod 
  AND vo.titol = asi.votacio AND vo.data_obertura > ins.data_ins;
  IF num = 0 THEN
    tex_error := ''La inscripci del Ciutad: ''||cod||'' s fora de termini.'';
    RAISE excepcio;
  END IF;
         
  --- Registrem el Vot
  INSERT INTO VOT(votacio, ciutada, opcio, data_vot) VALUES (vot, cod, opc, da1);
  
  --- Actualitzem les dades estadstiques
  --- Votacions que participa ciutad
  UPDATE CIUTADA SET n_vot_part = n_vot_part + 1 WHERE codi = cod;
  --- Vots a una Votaci
  UPDATE VOTACIO SET total_vots = total_vots + 1 WHERE titol = vot;
  --- Participaci a Votaci
  --- Actualitzem el nmero de votants inscrits dins el termini
  UPDATE VOTACIO 
  SET total_ciutadans = (SELECT COUNT(ins.ciutada) 
                          FROM ASSIGNACIO asi, VOTACIO vo, INSCRIPCIO ins
                          WHERE vo.titol = vot AND asi.votacio = vo.titol AND
                                asi.cens = ins.cens AND ins.data_ins < vo.data_obertura)
  WHERE titol = vot;
  UPDATE VOTACIO 
  SET participacio = (SELECT (total_vots / total_ciutadans) * 100 FROM VOTACIO WHERE titol = vot),
      abstencio = (SELECT ((total_ciutadans - total_vots) / total_ciutadans) * 100 FROM VOTACIO WHERE titol = vot)
  WHERE titol = vot;
    
  --- Vots absoluts de l''Opci escollida
  UPDATE OPCIO SET vots_absoluts = vots_absoluts + 1 WHERE codi = opc;
  --- Vots relatius a l''Opci
  --- Obrim el cursor de codis
  OPEN C1;
  LOOP
    --- Fem un recorregut per tots ells
    FETCH C1 INTO num;
    EXIT WHEN C1%NOTFOUND;
    --- Actualitzem els Vots relatius a les Opcions
    UPDATE OPCIO opc SET vots_relatius = (SELECT (op.vots_absoluts / vo.total_vots) * 100 
                                       FROM OPCIO op, VOTACIO vo 
                                       WHERE op.votacio = vot AND vo.titol = op.votacio AND op.codi = num)
    WHERE opc.votacio = vot AND opc.codi = num;  
  END LOOP;
  --- Tanquem el cursor
  CLOSE C1;
  
  --- Participacio al cens del Ciutad assignat a la Votaci en curs
  SELECT ins.cens INTO auxiliar FROM ASSIGNACIO ass, INSCRIPCIO ins
  WHERE ins.ciutada = cod AND ins.cens = ass.cens AND ass.votacio = vot;
  --- Actualitzaci de les dades
  UPDATE ASSIGNACIO SET vots = vots + 1 
  WHERE cens = auxiliar AND votacio = vot;
  UPDATE ASSIGNACIO SET participacio = vots / votants * 100 
  WHERE cens = auxiliar AND votacio = vot;
  UPDATE ASSIGNACIO SET abstencio = (votants - vots) / votants * 100 
  WHERE cens = auxiliar AND votacio = vot;
  
  --- Diferncia entre Opcions
  UPDATE VOTACIO SET dif_opcions = (SELECT MAX(vots_relatius) - MIN(vots_relatius) 
                                    FROM OPCIO WHERE votacio = vot GROUP BY votacio)
  WHERE titol = vot;
  
  --- Esbrinem la Votaci amb ms participaci
  SELECT titol INTO auxiliar 
  FROM (SELECT titol FROM VOTACIO WHERE participacio = (SELECT MAX(participacio) FROM VOTACIO)) 
  WHERE ROWNUM = 1;
  --- Actualitzem la dada estadstica
  UPDATE GRUP_EST SET mes_participacio = auxiliar WHERE codi = ''VotaMesParticipacio'';
  
  --- Esbrinem la Votaci amb ms diferencia de resultat entre les opcions ms i menys votades
  SELECT titol INTO auxiliar 
  FROM (SELECT titol, dif_opcions FROM VOTACIO WHERE estat IN (''Oberta'',''Tancada'') ORDER BY dif_opcions DESC) 
  WHERE ROWNUM = 1;
  --- Actualitzem la dada estadstica
  UPDATE GRUP_EST SET mes_dif_opc = auxiliar WHERE codi = ''VotaMesDifOpcions'';
  
  --- Esbrinem la Votaci amb menys diferencia entre opci maxima i mnima
  SELECT titol INTO auxiliar 
  FROM (SELECT titol, dif_opcions FROM VOTACIO WHERE estat IN (''Oberta'',''Tancada'') ORDER BY dif_opcions ASC) 
  WHERE ROWNUM = 1;
  --- Actualitzem la dada estadstica
  UPDATE GRUP_EST SET menys_dif_opc = auxiliar WHERE codi = ''VotaMenysDifOpcions'';
  
  --- Actualitzem el nmero mxim de votacions que ha participat una mateixa persona i el Pas
  UPDATE GRUP_EST SET 
    ciutada = (SELECT codi 
			   FROM (SELECT codi FROM CIUTADA 
					 WHERE n_vot_part = (SELECT MAX(n_vot_part) FROM CIUTADA)) 
			   WHERE ROWNUM = 1),
    pais = (SELECT estat 
			FROM (SELECT estat FROM CIUTADA 
				  WHERE n_vot_part = (SELECT MAX(n_vot_part) FROM CIUTADA)) 
			WHERE ROWNUM = 1),
    n_votacions = (SELECT n_vot_part 
				   FROM (SELECT n_vot_part FROM CIUTADA 
						 WHERE n_vot_part = (SELECT MAX(n_vot_part) FROM CIUTADA)) 
				   WHERE ROWNUM = 1)
  WHERE codi = ''NMaxVota1Persona'';
  
  --- Actualitzem la votaci amb ms participaci d''aquest any
  SELECT COUNT(anys) INTO num FROM GRUP_EST WHERE anys = (SELECT EXTRACT(YEAR FROM da1) FROM DUAL);
  --- Esbrinem quina s la votaci amb ms participaci de l''any
  SELECT titol INTO auxiliar FROM (SELECT titol  
                                   FROM VOTACIO 
                                   WHERE participacio = (SELECT MAX(participacio) FROM VOTACIO 
                                          WHERE (SELECT EXTRACT(YEAR FROM data_tancament) FROM DUAL) = 
                                                (SELECT EXTRACT(YEAR FROM da1) FROM DUAL)))
  WHERE ROWNUM = 1;
  --- Si ja s''ha fet alguna votaci aquest any
  IF num = 1 THEN
    UPDATE GRUP_EST
      SET vot_mes_part = auxiliar
    WHERE anys = (SELECT EXTRACT(YEAR FROM da1) FROM DUAL);
    
  --- Si no creem l''entrada nova amb l''any com a codi
  ELSE
  INSERT INTO GRUP_EST (codi, anys, vot_mes_part)
  VALUES (
  (SELECT TO_CHAR( EXTRACT(YEAR FROM da1)) FROM DUAL), 
  (SELECT EXTRACT(YEAR FROM da1) FROM DUAL), 
   auxiliar
  );
  END IF;
  
  --- Aprofitem per actualitzar la quantitat de ciutadans que mai han votat aquest any
  UPDATE GRUP_EST
    SET n_mai_vot = (SELECT (SELECT COUNT(codi) FROM CIUTADA 
                     WHERE codi NOT IN(SELECT ciutada FROM VOT WHERE 
                                      (SELECT EXTRACT(YEAR FROM data_vot) FROM DUAL) = 
                                      (SELECT EXTRACT(YEAR FROM da1) FROM DUAL))) / 
                                      (SELECT COUNT(codi) FROM ciutada) * 100 
                     FROM DUAL)
  WHERE anys = (SELECT EXTRACT(YEAR FROM da1) FROM DUAL); 
  
  --- Actualitzem la localitat que ms vots han ems els seus Ciutadans aquest any
  UPDATE GRUP_EST
    SET local_mes_vots= (SELECT nomc FROM (SELECT lo.nom AS nomc, COUNT(ciutada) 
                                         FROM VOT vo, CIUTADA ci, LOCALITAT lo 
                                         WHERE (SELECT EXTRACT(YEAR FROM vo.data_vot) FROM DUAL) = 
                                               (SELECT EXTRACT(YEAR FROM da1) FROM DUAL) 
                                                AND vo.ciutada = ci.codi AND ci.localitat = lo.codi 
                                                GROUP BY lo.nom ORDER BY COUNT(ciutada) DESC)
                        WHERE ROWNUM = 1)
  WHERE anys = (SELECT EXTRACT(YEAR FROM da1) FROM DUAL); 
            
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Votar'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Errors personalitzats
  WHEN excepcio THEN
    n_error := SQLCODE;
    --- tex_error := ''El Ciutad no figura als Censos.'';
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Votar'', entrades, RSP);      
  --- Tractament de la resta d''errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Votar'', entrades, RSP);  
END;';
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---18----- Procediment per generar els llistats --------------------------------
--------------------------------------------------------------------------------
EXECUTE IMMEDIATE 'CREATE OR REPLACE PROCEDURE LLISTAT(tit IN VARCHAR2, 
pais IN VARCHAR2, cod IN NUMBER, RSP OUT VARCHAR2) 
IS
  n_error NUMBER; --- Nmero d''error
  tex_error VARCHAR2(100); --- Text d''error
  entrades VARCHAR(200); --- Per Parmetres d''entrada
  excepcio EXCEPTION; --- Nom de l''excepci
  CURSOR C1 IS SELECT DISTINCT v.titol, v.data_obertura, v.data_tancament, v.total_ciutadans, v.estat, v.participacio
               FROM VOTACIO v, ASSIGNACIO asi
               WHERE v.titol = asi.votacio AND 
                    asi.cens IN(SELECT cens FROM ASSIGNACIO ag, CENS cg WHERE ag.cens = cg.nom AND cg.ambit = ''Global''
                    UNION
                    SELECT cens FROM ASSIGNACIO ae, CENS_ESTAT ce WHERE ae.cens = ce.nom AND ce.estat = pais
                    UNION
                    SELECT cens FROM ASSIGNACIO ar, CENS_REGIO cr WHERE ar.cens = cr.nom AND cr.estat = pais
                    UNION
                    SELECT cens FROM ASSIGNACIO al, CENS_LOCAL cl WHERE al.cens = cl.nom AND cl.estat = pais)
               ORDER BY v.data_obertura;
  CURSOR C2 IS SELECT titol, data_obertura, data_tancament, dif_opcions FROM VOTACIO WHERE estat IN (''Oberta'',''Tancada'') 
               ORDER BY dif_opcions DESC;
  CURSOR C3 IS SELECT titol, data_obertura, data_tancament, dif_opcions FROM VOTACIO WHERE estat IN (''Oberta'',''Tancada'') 
               ORDER BY dif_opcions ASC;
  CURSOR C4 IS SELECT cens FROM INSCRIPCIO WHERE ciutada = cod;
  CURSOR C5 IS  SELECT votacio, ''Si'' AS Participat FROM VOT WHERE ciutada = cod
                UNION
                SELECT DISTINCT titol, ''No'' AS Participat FROM VOTACIO 
                WHERE estat = ''Oberta'' OR estat = ''Tancada'' AND titol NOT IN(SELECT votacio FROM VOT WHERE ciutada = cod);
  num NUMBER;
  v1 VARCHAR2(40);
  v2 VARCHAR2(40);
  v3 DATE;
  v4 DATE;
  v5 NUMBER(5,2);
  v6 NUMBER(5,2);
  v7 VARCHAR2(40);
  v8 NUMBER(5,2);
  v9 VARCHAR2(40);
  v10 VARCHAR2(40);
  v11 VARCHAR2(40);

BEGIN
  
  --- Parmetres d''entrada, introducci de dades
  entrades := tit || '', '' || pais || '', '' || cod;
  
  --- Comprovacions de seguretat
  IF tit IS NULL THEN 
    tex_error := ''No s''''ha escollit cap llistat.'';
    RAISE excepcio; 
  END IF;
  IF tit NOT IN(''a'',''A'',''b'',''B'',''c'',''C'',''d'',''D'',''e'',''E'') THEN
    tex_error := ''El llistat: ''||tit||'' no est definit.'';
    RAISE excepcio;
  END IF;
  
  --- Definici del llistat a(A)
  IF tit IN (''a'', ''A'') THEN
    --- Comprovacions
    IF pais IS NULL THEN 
      tex_error := ''No es permeten valors buits com a Pas.'';
      RAISE excepcio; 
    END IF;
    SELECT COUNT(codi) INTO num FROM ESTAT WHERE codi = pais;
    IF num = 0 THEN 
      tex_error := ''No existeix cap Pas amb el codi: ''||pais||''.'';
      RAISE excepcio; 
    END IF;
    
    --- Obrim el cursor de codis
    OPEN C1;
    --- Capalera del llistat
      DBMS_OUTPUT.PUT_LINE(RPAD(''   VOTACI'',25)||RPAD(''  INICI'',10)||RPAD(''   FI'',10)
      ||LPAD(''CIUTADANS'',10)||'' ''|| RPAD('' ESTAT'',8)||RPAD(''PARTICIPACI'',12)||'' ''
	  || RPAD('' OPCI GUANYADORA'',25)||RPAD(''% VOTS'',7));
      DBMS_OUTPUT.PUT_LINE(RPAD(''-'',109,''-''));
    LOOP
      --- Fem un recorregut per tots ells
      FETCH C1 INTO v1, v3, v4, num, v2, v5;
      
      --- Si la Votaci s tancada
      IF v2 = ''Tancada'' THEN
        v9 := TO_CHAR(v5);
        --- Opcio ms votada
        SELECT v02, v06 INTO v10, v6 FROM (SELECT descripcio AS v02, vots_relatius AS v06 FROM OPCIO 
        WHERE votacio = v1 AND 
              vots_relatius = (SELECT MAX(vots_relatius)FROM OPCIO WHERE votacio = v1)) 
        WHERE ROWNUM = 1;
        v11 := TO_CHAR(v6);
      --- Si no ho s
      ELSE
        v9 := ''-'';
        v10 := ''-'';
        v11 := ''-'';
      END IF;
      
      EXIT WHEN C1%NOTFOUND;
      --- Presentem el llistat
      DBMS_OUTPUT.PUT_LINE(RPAD(v1,25)||RPAD(TO_CHAR(v3),10)||RPAD(TO_CHAR(v4),10)
      ||LPAD(num,10)||'' ''|| RPAD(v2,8)||LPAD(v9,12)||'' ''|| RPAD(v10,25)||LPAD(v11,7));  
    END LOOP;
    --- Tanquem el cursor
    CLOSE C1;
    --- Fi del llistat
    DBMS_OUTPUT.PUT_LINE(RPAD(''-'',109,''-''));
  END IF;
  
  --- Definici del llistat b(B)
  IF tit IN (''b'', ''B'') THEN
    num := 0;
    --- Obrim el cursor de codis
    OPEN C2;
    --- Capalera del llistat
    DBMS_OUTPUT.PUT_LINE(RPAD(''      VOTACI'',25)||RPAD(''  INICI'',10)||RPAD(''   FI'',10)
    ||RPAD('' OPCI GUANYADORA'',25)||RPAD(''% VOTS'',8)||RPAD('' OPCI PERDEDORA'',25)||RPAD(''% VOTS'',8)||RPAD(''% MARGE'',8));
    DBMS_OUTPUT.PUT_LINE(RPAD(''-'',118,''-''));
    LOOP
      --- Fem un recorregut per tots ells
      FETCH C2 INTO v1, v3, v4, v5;
      
      --- Opcio ms votada
      SELECT v02, v06 INTO v2, v6 FROM (SELECT descripcio AS v02, vots_relatius AS v06 FROM OPCIO 
      WHERE votacio = v1 AND 
            vots_relatius = (SELECT MAX(vots_relatius)FROM OPCIO WHERE votacio = v1)) 
      WHERE ROWNUM = 1;
      
      --- Opcio menys votada
      SELECT v02, v06 INTO v7, v8 FROM (SELECT descripcio AS v02, vots_relatius AS v06 FROM OPCIO 
      WHERE votacio = v1 AND 
            vots_relatius = (SELECT MIN(vots_relatius)FROM OPCIO WHERE votacio = v1)) 
      WHERE ROWNUM = 1;
      
      EXIT WHEN C2%NOTFOUND OR num = 10;
      --- Presentem el llistat
      DBMS_OUTPUT.PUT_LINE(RPAD(v1,25)||RPAD(TO_CHAR(v3),10)||RPAD(TO_CHAR(v4),10)
      ||RPAD(v2,25)||RPAD(v6,8)||RPAD(v7,25)||RPAD(v8,8)||RPAD(v5,8)); 
      num := num + 1;
    END LOOP;
    --- Tanquem el cursor
    CLOSE C2;
    --- Fi del llistat
    DBMS_OUTPUT.PUT_LINE(RPAD(''-'',118,''-''));
  END IF;
  
  --- Definici del llistat c(C)
  IF tit IN (''c'', ''C'') THEN
    num := 0;
    --- Obrim el cursor de codis
    OPEN C3;
    --- Capalera del llistat
    DBMS_OUTPUT.PUT_LINE(RPAD(''      VOTACI'',25)||RPAD(''  INICI'',10)||RPAD(''   FI'',10)
    ||RPAD('' OPCI GUANYADORA'',25)||RPAD(''% VOTS'',8)||RPAD('' OPCI PERDEDORA'',25)||RPAD(''% VOTS'',8)||RPAD(''% MARGE'',8));
    DBMS_OUTPUT.PUT_LINE(RPAD(''-'',118,''-''));
    LOOP
      --- Fem un recorregut per tots ells
      FETCH C3 INTO v1, v3, v4, v5;
      
      --- Opcio ms votada
      SELECT v02, v06 INTO v2, v6 FROM (SELECT descripcio AS v02, vots_relatius AS v06 FROM OPCIO 
      WHERE votacio = v1 AND 
            vots_relatius = (SELECT MAX(vots_relatius)FROM OPCIO WHERE votacio = v1)) 
      WHERE ROWNUM = 1;
      
      --- Opcio menys votada
      SELECT v02, v06 INTO v7, v8 FROM (SELECT descripcio AS v02, vots_relatius AS v06 FROM OPCIO 
      WHERE votacio = v1 AND 
            vots_relatius = (SELECT MIN(vots_relatius)FROM OPCIO WHERE votacio = v1)) 
      WHERE ROWNUM = 1;
      
      EXIT WHEN C3%NOTFOUND OR num = 10;
      --- Presentem el llistat
      DBMS_OUTPUT.PUT_LINE(RPAD(v1,25)||RPAD(TO_CHAR(v3),10)||RPAD(TO_CHAR(v4),10)
      ||RPAD(v2,25)||RPAD(v6,8)||RPAD(v7,25)||RPAD(v8,8)||RPAD(v5,8)); 
      num := num + 1;
    END LOOP;
    --- Tanquem el cursor
    CLOSE C3;
    --- Fi del llistat
    DBMS_OUTPUT.PUT_LINE(RPAD(''-'',118,''-''));
  END IF;
  
  --- Definici del llistat d(D)
  IF tit IN (''d'', ''D'') THEN
    --- Comprovacions
    IF cod IS NULL THEN 
      tex_error := ''No es permeten valors buits com a Ciutad.'';
      RAISE excepcio; 
    END IF;
    SELECT COUNT(codi) INTO num FROM CIUTADA WHERE codi = cod;
    IF num = 0 THEN 
      tex_error := ''No existeix cap Ciutad amb el codi: ''||cod||''.'';
      RAISE excepcio; 
    END IF;
    
    --- Obrim el cursor de codis
    OPEN C4;
    --- Capalera del llistat
    DBMS_OUTPUT.PUT_LINE(RPAD(''     CENS'',16));
    DBMS_OUTPUT.PUT_LINE(RPAD(''-'',16,''-''));
    LOOP
      --- Fem un recorregut per tots ells
      FETCH C4 INTO v1;
      EXIT WHEN C4%NOTFOUND;
      --- Presentem el llistat
      DBMS_OUTPUT.PUT_LINE(RPAD(v1,16));  
    END LOOP;
    --- Tanquem el cursor
    CLOSE C4;
    --- Fi del llistat
    DBMS_OUTPUT.PUT_LINE(RPAD(''-'',16,''-''));
  END IF;
  
  --- Definici del llistat e(E)
  IF tit IN (''e'', ''E'') THEN
    --- Comprovacions
    IF cod IS NULL THEN 
      tex_error := ''No es permeten valors buits com a Ciutad.'';
      RAISE excepcio; 
    END IF;
    SELECT COUNT(codi) INTO num FROM CIUTADA WHERE codi = cod;
    IF num = 0 THEN 
      tex_error := ''No existeix cap Ciutad amb el codi: ''||cod||''.'';
      RAISE excepcio; 
    END IF;
    
    --- Obrim el cursor de codis
    OPEN C5;
    --- Capalera del llistat
    DBMS_OUTPUT.PUT_LINE(RPAD(''      VOTACIO'',25)||RPAD(''HA PARTICIPAT'',15));
    DBMS_OUTPUT.PUT_LINE(RPAD(''-'',38,''-''));
    LOOP
      --- Fem un recorregut per tots ells
      FETCH C5 INTO v1, v2;
      EXIT WHEN C5%NOTFOUND;
      --- Presentem el llistat
      DBMS_OUTPUT.PUT_LINE(RPAD(v1,25)||RPAD(v2,25));  
    END LOOP;
    --- Tanquem el cursor
    CLOSE C5;
    --- Fi del llistat
    DBMS_OUTPUT.PUT_LINE(RPAD(''-'',38,''-''));
  END IF;
  
  --- Si no hi ha cap error posem OK al RSP(parmetre de sortida)
  RSP := ''OK'';
  --- Insertem una fila a la taula Logs 
  INSERT 
  INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
  VALUES(SYSDATE, ''Llistat'', entrades, RSP);
  
--- Definici d''excepcions
EXCEPTION
  --- Tractament dels errors personalitzat
  WHEN excepcio THEN
    n_error := SQLCODE;
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Llistat'', entrades, RSP);
  --- Tractament de la resta dels errors
  WHEN OTHERS THEN
    n_error := SQLCODE;
    tex_error := SUBSTR(SQLERRM, 11 , 100);
    --- Si hi ha hagut un error posem a RSP(Sortida) el nmero i text del mateix
    RSP := ''ERROR: '' || n_error || '': '' || tex_error;
    --- Insertem una fila a la taula Logs amb l''error
    INSERT 
    INTO TAULA_LOGS(data_crida, nom_procediment, par_entrada, par_sortida) 
    VALUES(SYSDATE, ''Llistat'', entrades, RSP);
END;';

END;
/