--------------------------------------------------------------------------------
---- Inserci de Dades per el test del sistema ---------------------------------
--------------------------------------------------------------------------------

DECLARE

RSP VARCHAR2(200);
CURSOR C1 IS SELECT codi FROM ESTAT;
CURSOR C2 IS SELECT estat, codi FROM REGIO;
CURSOR C3 IS SELECT estat, regio, codi FROM LOCALITAT;
CURSOR C4 IS SELECT codi FROM CIUTADA ORDER BY codi;
v1 VARCHAR2(50);
v2 VARCHAR2(50);
v3 VARCHAR2(50);
dn DATE;
num NUMBER;
id1 NUMBER;

BEGIN

--------------------------------------------------------------------------------
----- Esborrem el disparador que protegeig els Vots ----------------------------
--------------------------------------------------------------------------------

EXECUTE IMMEDIATE 'DROP TRIGGER no_edit_borr_vot';

--------------------------------------------------------------------------------
----- Buidem el contingut de totes les taules ----------------------------------
--------------------------------------------------------------------------------

  EXECUTE IMMEDIATE 'DELETE FROM GRUP_EST'; 
  EXECUTE IMMEDIATE 'DELETE FROM EST_PAIS_ANY';
  EXECUTE IMMEDIATE 'DELETE FROM EST_CENS_ANY';
  EXECUTE IMMEDIATE 'DELETE FROM TAULA_LOGS';
  EXECUTE IMMEDIATE 'DELETE FROM VOT';  
  EXECUTE IMMEDIATE 'DELETE FROM ASSIGNACIO';
  EXECUTE IMMEDIATE 'DELETE FROM INSCRIPCIO';
  EXECUTE IMMEDIATE 'DELETE FROM CENS_LOCAL';
  EXECUTE IMMEDIATE 'DELETE FROM CENS_REGIO';
  EXECUTE IMMEDIATE 'DELETE FROM CENS_ESTAT';
  EXECUTE IMMEDIATE 'DELETE FROM CENS';
  EXECUTE IMMEDIATE 'DELETE FROM CIUTADA';    
  EXECUTE IMMEDIATE 'DELETE FROM VOTACIO';  
  EXECUTE IMMEDIATE 'DELETE FROM OPCIO';
  EXECUTE IMMEDIATE 'DELETE FROM LOCALITAT WHERE codi = ''ES_CAT_TEIA''';
  EXECUTE IMMEDIATE 'DELETE FROM CENS WHERE nom = ''Loc_ES_CAT_TEIA''';

--------------------------------------------------------------------------------
--- Eliminem i tornem a crear les seqncies per qu tornin a comenar de 0 ----
--------------------------------------------------------------------------------

  --- Les eliminem:
  EXECUTE IMMEDIATE 'DROP SEQUENCE SEC_TAULA_LOGS';
  EXECUTE IMMEDIATE 'DROP SEQUENCE SEC_OPCIO';
  
  --- Les tornem a crear:
  EXECUTE IMMEDIATE 'CREATE SEQUENCE SEC_TAULA_LOGS INCREMENT BY 1 START WITH 1';
  EXECUTE IMMEDIATE 'CREATE SEQUENCE SEC_OPCIO INCREMENT BY 1 START WITH 1';
  
--------------------------------------------------------------------------------
------ Tornem a crear el trigger de seguretat pe el Vot ------------------------
--------------------------------------------------------------------------------

EXECUTE IMMEDIATE 'CREATE OR REPLACE TRIGGER no_edit_borr_vot
  BEFORE UPDATE OR DELETE ON VOT
  FOR EACH ROW
BEGIN
  raise_application_error(-20000, ''ERROR: GREU!!. No es permet editar ni esborrar cap vot registrat'');  
END no_edit_borr_vot;';

--------------------------------------------------------------------------------
--- Incialitzaci mnima de la taula Cens: 
--- Cens Global al qual pertanyen tots els Ciutadans ---------------------------
--------------------------------------------------------------------------------

INSERT INTO CENS VALUES ('Europa', 'Global', 0);
--- Tamb es pot realitzar aquesta alta amb el procediment creat a tal efecte
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-----| Iniciaci de la taula Grup_Est |-----------------------------------------
--------------------------------------------------------------------------------

INSERT INTO GRUP_EST (codi)
  SELECT 'VotaMesParticipacio' FROM DUAL
  UNION ALL
  SELECT 'VotaMesDifOpcions' FROM DUAL
  UNION ALL
  SELECT 'VotaMenysDifOpcions' FROM DUAL
  UNION ALL
  SELECT 'NMaxVota1Persona' FROM DUAL;

--------------------------------------------------------------------------------
---- Creem els Censos de totes les Localitats, Estats i Regions del sistema ----
--------------------------------------------------------------------------------
  --- Obrim el cursor d'Estats
  OPEN C1;
  LOOP
    --- Fem un recorregut per tots ells
    FETCH C1 INTO v1;
    --- Si no hi han ms dades sortim
    EXIT WHEN C1%NOTFOUND;
    ALTA_CENS('Estatal', v1, v2, v3, RSP);  
  END LOOP;
  --- Tanquem el cursor
  CLOSE C1;
  
  --- Obrim el cursor de Regions
  OPEN C2;
  LOOP
    --- Fem un recorregut per tots ells
    FETCH C2 INTO v1, v2;
    --- Si no hi han ms dades sortim
    EXIT WHEN C2%NOTFOUND;
    ALTA_CENS('Regional', v1, v2, v3, RSP);  
  END LOOP;
  --- Tanquem el cursor
  CLOSE C2;
  
  --- Obrim el cursor de Localitats
  OPEN C3;
  LOOP
    --- Fem un recorregut per tots elles
    FETCH C3 INTO v1, v2, v3;
    --- Si no hi han ms dades sortim
    EXIT WHEN C3%NOTFOUND;
    ALTA_CENS('Local', v1, v2, v3, RSP);  
  END LOOP;
  --- Tanquem el cursor
  CLOSE C3;

--------------------------------------------------------------------------------
-------- Inserim vries Votacions amb les seves Opcions ------------------------
--------------------------------------------------------------------------------

  --- Votaci 1 (Completa)
  ------------------------
  ALTA_VOTACIO('Constitucio Europea', 
               '10/05/2009', 
               '18/05/2009', 
               '22/05/2009', 
               'Xavier Lopez Camps', 
               'Aprova el text de la proposta de Costituci Europea en la seva totalitat?. Si, No o Vot en blanc', 
               'http://www.europa.eu/consultes/constitucio', RSP);

  --- Opcions Votaci 1 (x3)
  ALTA_OPCIO('Constitucio Europea', 'Si', RSP); 
  ALTA_OPCIO('Constitucio Europea', 'No', RSP); 
  ALTA_OPCIO('Constitucio Europea', 'Vot en blanc', RSP);

  --- Votaci 2 (Completa)
  ------------------------
  ALTA_VOTACIO('Riera Premia', 
               '10/06/2009', 
               '18/06/2009', 
               '22/06/2009', 
               'Jordi Lopez Camps', 
               'Est d''acord amb el projecte de Modificaci de la Riera de Premi?. Si, No o Vot en blanc', 
               'http://www.europa.eu/consultes/riera_premia', RSP);

  --- Opcions Votaci 2 (x3)
  ALTA_OPCIO('Riera Premia', 'Si', RSP); 
  ALTA_OPCIO('Riera Premia', 'No', RSP); 
  ALTA_OPCIO('Riera Premia', 'Vot en blanc', RSP);

  --- Votaci 3 (Completa)
  ------------------------
  ALTA_VOTACIO('Europarlament 2010', 
               '10/06/2010', 
               '18/06/2010', 
               '22/06/2010', 
               'Jordi Mas Camps', 
               'Eleccions al Parlament Europeu 2010. PSOE, PP, CIU, ERC, ICV o Vot en blanc', 
               'http://www.europa.eu/consultes/parlament_2010', RSP);

  --- Opcions Votaci 3 (x6)
  ALTA_OPCIO('Europarlament 2010', 'PSOE', RSP); 
  ALTA_OPCIO('Europarlament 2010', 'PP', RSP);
  ALTA_OPCIO('Europarlament 2010', 'CIU', RSP);
  ALTA_OPCIO('Europarlament 2010', 'ERC', RSP);
  ALTA_OPCIO('Europarlament 2010', 'ICV', RSP);
  ALTA_OPCIO('Europarlament 2010', 'Vot en blanc', RSP);

  --- Votaci 4 (Completa)
  ------------------------
  ALTA_VOTACIO('Corredor Mediterrani', 
               '10/06/2010', 
               '18/06/2010', 
               '22/06/2010', 
               'Jordi Mas Camps', 
               'Est d''acord en prioritzar el Corredor Mediterrani. Si, immediatament; Si, ms endevant; No, mai o Vot en blanc', 
               'http://www.europa.eu/consultes/corredor_mediterrani', RSP);

  --- Opcions Votaci 4 (x4)
  ALTA_OPCIO('Corredor Mediterrani', 'Si, immediatament', RSP); 
  ALTA_OPCIO('Corredor Mediterrani', 'Si, ms endevant', RSP);
  ALTA_OPCIO('Corredor Mediterrani', 'No, mai', RSP);
  ALTA_OPCIO('Corredor Mediterrani', 'Vot en blanc', RSP);

  --- Votaci 5 (Completa)
  ------------------------
  ALTA_VOTACIO('Corredor Central', 
               '10/07/2010', 
               '18/07/2010', 
               '22/07/2010', 
               'Pep Mass Camps', 
               'Est d''acord en prioritzar el Corredor Central. Si, immediatament; Si, ms endevant; No, mai o Vot en blanc', 
               'http://www.europa.eu/consultes/corredor_central', RSP);

  --- Opcions Votaci 5 (x4)
  ALTA_OPCIO('Corredor Central', 'Si, immediatament', RSP); 
  ALTA_OPCIO('Corredor Central', 'Si, ms endevant', RSP);
  ALTA_OPCIO('Corredor Central', 'No, mai', RSP);
  ALTA_OPCIO('Corredor Central', 'Vot en blanc', RSP);

  --- Votaci 6 (Completa)
  ------------------------
  ALTA_VOTACIO('Depuradora Maresme', 
               '10/07/2011', 
               '18/07/2011', 
               '22/07/2011', 
               'Llus Mass Campos', 
               'Est d''acord en el projecte de Depuradora per el Maresme. Si, immediatament; Si, ms endevant; No, mai o Vot en blanc', 
               'http://www.europa.eu/consultes/depuradora_maresme', RSP);

  --- Opcions Votaci 6 (x4)
  ALTA_OPCIO('Depuradora Maresme', 'Si, immediatament', RSP); 
  ALTA_OPCIO('Depuradora Maresme', 'Si, ms endevant', RSP);
  ALTA_OPCIO('Depuradora Maresme', 'No, mai', RSP);
  ALTA_OPCIO('Depuradora Maresme', 'Vot en blanc', RSP);

  --- Votaci 7 (Completa)
  ------------------------
  ALTA_VOTACIO('Terme Barcelona', 
               '10/07/2011', 
               '18/07/2011', 
               '22/07/2011', 
               'Llus Mass Campos', 
               'Est d''acord l''ampliaci del terme municipal de Barcelona. Si; Indiferent; No o Vot en blanc', 
               'http://www.europa.eu/consultes/terme_barcelona', RSP);

  --- Opcions Votaci 7 (x4)
  ALTA_OPCIO('Terme Barcelona', 'Si', RSP); 
  ALTA_OPCIO('Terme Barcelona', 'No', RSP);
  ALTA_OPCIO('Terme Barcelona', 'Indiferent', RSP);
  ALTA_OPCIO('Terme Barcelona', 'Vot en blanc', RSP);

  --- Votaci 8 (Completa)
  ------------------------
  ALTA_VOTACIO('Relacio ES FR', 
               '10/07/2011', 
               '18/07/2011', 
               '22/07/2011', 
               'Elisa Mass Campos', 
               'Est d''acord en l''ampliaci de les relacions bilaterals entre Frana i Espanya. Si; No o Vot en blanc', 
               'http://www.europa.eu/consultes/relacions_bilaterals/FR_ES', RSP);

  --- Opcions Votaci 8 (x3)
  ALTA_OPCIO('Relacio ES FR', 'Si', RSP); 
  ALTA_OPCIO('Relacio ES FR', 'No', RSP);
  ALTA_OPCIO('Relacio ES FR', 'Vot en blanc', RSP);

  --- Votaci 9 (Completa)
  ------------------------
  ALTA_VOTACIO('Relacio ES PT', 
               '10/07/2011', 
               '18/07/2011', 
               '22/07/2011', 
               'Carme Mass Campos', 
               'Est d''acord en l''ampliaci de les relacions bilaterals entre Portugal i Espanya. Si; No o Vot en blanc', 
               'http://www.europa.eu/consultes/relacions_bilaterals/PT_ES', RSP);

  --- Opcions Votaci 9 (x3)
  ALTA_OPCIO('Relacio ES PT', 'Si', RSP); 
  ALTA_OPCIO('Relacio ES PT', 'No', RSP);
  ALTA_OPCIO('Relacio ES PT', 'Vot en blanc', RSP);

  --- Votaci 10 (Completa)
  -------------------------
  ALTA_VOTACIO('Millora Platges', 
               '10/07/1999', 
               '18/07/1999', 
               '22/07/1999', 
               'Carme Mass Mass', 
               'Est d''acord en millorar les infraestructures de les platges del Mediterrani. Si; No o Vot en blanc', 
               'http://www.europa.eu/consultes/millora_platges/mediterrani', RSP);

  --- Opcions Votaci 10 (x3)
  ALTA_OPCIO('Millora Platges', 'Si', RSP); 
  ALTA_OPCIO('Millora Platges', 'No', RSP);
  ALTA_OPCIO('Millora Platges', 'Vot en blanc', RSP);

  --- Votaci 11 (Completa)
  -------------------------
  ALTA_VOTACIO('Entrar Euro', 
               '10/07/2000', 
               '18/07/2000', 
               '22/07/2000', 
               'Carme Mass Mass', 
               'Est d''acord que el seu Pas accepti l''Euro com a nova moneda. Si; No o Vot en blanc', 
               'http://www.europa.eu/consultes/euro/incorporacio', RSP);

  --- Opcions Votaci 11 (x3)
  ALTA_OPCIO('Entrar Euro', 'Si', RSP); 
  ALTA_OPCIO('Entrar Euro', 'No', RSP);
  ALTA_OPCIO('Entrar Euro', 'Vot en blanc', RSP);

  --- Votaci 12 (Completa)
  -------------------------
  ALTA_VOTACIO('Sortir Euro', 
               '10/12/2011', 
               '18/12/2011', 
               '22/12/2011', 
               'Carme Mass Mass', 
               'Est d''acord que el seu Pas torni a la seva antiga moneda. Si; No o Vot en blanc', 
               'http://www.europa.eu/consultes/euro/sortida', RSP);

  --- Opcions Votaci 12 (x3)
  ALTA_OPCIO('Sortir Euro', 'Si', RSP); 
  ALTA_OPCIO('Sortir Euro', 'No', RSP);
  ALTA_OPCIO('Sortir Euro', 'Vot en blanc', RSP);

  --- Votaci 13 (Completa)
  -------------------------
  ALTA_VOTACIO('Premia Mar i Dalt', 
               '10/02/2011', 
               '18/02/2011', 
               '22/02/2011', 
               'Carme Mass Mass', 
               'Est d''acord que s''unifiquin els termes municipals de Premi de Mar i de Premi de Dalt. Si; No o Vot en blanc', 
               'http://www.europa.eu/consultes/termes_municipals/premia_mar_dalt', RSP);

  --- Opcions Votaci 13 (x3)
  ALTA_OPCIO('Premia Mar i Dalt', 'Si', RSP); 
  ALTA_OPCIO('Premia Mar i Dalt', 'No', RSP);
  ALTA_OPCIO('Premia Mar i Dalt', 'Vot en blanc', RSP);
  
  --- Votaci 14 (Completa)
  -------------------------
  ALTA_VOTACIO('Premia Mar Masnou', 
               '10/01/2011', 
               '18/01/2011', 
               '22/01/2011', 
               'Jlia Mass Mass', 
               'Est d''acord que s''unifiquin els termes municipals de Premi de Mar i d''El Masnou. Si; No o Vot en blanc', 
               'http://www.europa.eu/consultes/termes_municipals/premia_masnou', RSP);

  --- Opcions Votaci 14 (x3)
  ALTA_OPCIO('Premia Mar Masnou', 'Si', RSP); 
  ALTA_OPCIO('Premia Mar Masnou', 'No', RSP);
  ALTA_OPCIO('Premia Mar Masnou', 'Vot en blanc', RSP);

  --- Votaci 15 (Votaci amb una sola Opci)
  -------------------------------------------
  ALTA_VOTACIO('Obertura Atlantica', 
               '10/02/2011', 
               '18/02/2011', 
               '22/02/2011', 
               'Irene Juli Mass', 
               'Est d''acord que s''amplin les relacions amb Amrica Latina. Si; No o Vot en blanc', 
               'http://www.europa.eu/consultes/exteriors/america_latina', RSP);

  --- Opcions Votaci 15 (x1)
  ALTA_OPCIO('Obertura Atlantica', 'Si', RSP); 
  
  --- Votaci 16 (Completa)
  -------------------------
  ALTA_VOTACIO('Atlantic Nord', 
               '10/01/2011', 
               '18/01/2011', 
               '22/01/2011', 
               'Juli Mass Mass', 
               'Est d''acord que s''amplin les relacions amb Amrica del Nord. Si; No o Vot en blanc', 
               'http://www.europa.eu/consultes/exteriors/america_nord', RSP);

  --- Opcions Votaci 16 (x3)
  ALTA_OPCIO('Atlantic Nord', 'Si', RSP); 
  ALTA_OPCIO('Atlantic Nord', 'No', RSP);
  ALTA_OPCIO('Atlantic Nord', 'Vot en blanc', RSP);
  
  --- Votaci 17 (Completa)
  -------------------------
  ALTA_VOTACIO('Banc Central', 
               '10/08/2009', 
               '18/08/2009', 
               '22/08/2009', 
               'Xavier Lopez Camps', 
               'Aprova la creaci d''un Banc Central Europeu?. Si, No o Vot en blanc', 
               'http://www.europa.eu/consultes/banc_central', RSP);

  --- Opcions Votaci 17 (x3)
  ALTA_OPCIO('Banc Central', 'Si', RSP); 
  ALTA_OPCIO('Banc Central', 'No', RSP);
  ALTA_OPCIO('Banc Central', 'Vot en blanc', RSP);
  
  --- Votaci 18 (Completa)
  -------------------------
  ALTA_VOTACIO('Eurobons', 
               '10/08/2011', 
               '18/08/2011', 
               '22/08/2011', 
               'Xavier Lopez Camps', 
               'Est d''acord que el Banc Central Europeu pugui enetre Bons?. Si, No o Vot en blanc', 
               'http://www.europa.eu/consultes/banc_central/eurobons', RSP);

  --- Opcions Votaci 18 (x3)
  ALTA_OPCIO('Eurobons', 'Si', RSP); 
  ALTA_OPCIO('Eurobons', 'No', RSP);
  ALTA_OPCIO('Eurobons', 'Vot en blanc', RSP);
  
  --- Votaci 19 (Completa)
  -------------------------
  ALTA_VOTACIO('Mesures ES Crisis', 
               '10/08/2011', 
               '18/08/2011', 
               '22/08/2011', 
               'Jlia Lopez Garcia', 
               'Est d''acord amb les mesures econmiques proposades per la UE?. Si, No o Vot en blanc', 
               'http://www.europa.eu/consultes/espanya/mesures_economiques', RSP);

  --- Opcions Votaci 19 (x3)
  ALTA_OPCIO('Mesures ES Crisis', 'Si', RSP); 
  ALTA_OPCIO('Mesures ES Crisis', 'No', RSP);
  ALTA_OPCIO('Mesures ES Crisis', 'Vot en blanc', RSP);
  
  --- Votaci 20 (Completa)
  -------------------------
  ALTA_VOTACIO('Mesures PT Crisis', 
               '10/08/2011', 
               '18/08/2011', 
               '22/08/2011', 
               'Jlia Garcia Peres', 
               'Est d''acord amb les mesures econmiques proposades per la UE?. Si, No o Vot en blanc', 
               'http://www.europa.eu/consultes/portugal/mesures_economiques', RSP);

  --- Opcions Votaci 20 (x3)
  ALTA_OPCIO('Mesures PT Crisis', 'Si', RSP); 
  ALTA_OPCIO('Mesures PT Crisis', 'No', RSP);
  ALTA_OPCIO('Mesures PT Crisis', 'Vot en blanc', RSP);
  
  --- Votaci 21 (Completa)
  -------------------------
  ALTA_VOTACIO('Ampliacio UE', 
               '10/12/2001', 
               '18/12/2001', 
               '22/12/2001', 
               'Julia Frances Spencer', 
               'Est d''acord amb el projecte d''ampliaci de la UE?. Si, No o Vot en blanc', 
               'http://www.europa.eu/consultes/ue/ampliacio', RSP);

  --- Opcions Votaci 21 (x3)
  ALTA_OPCIO('Ampliacio UE', 'Si', RSP); 
  ALTA_OPCIO('Ampliacio UE', 'No', RSP);
  ALTA_OPCIO('Ampliacio UE', 'Vot en blanc', RSP);
  
  --- Votaci 22 (Completa)
  -------------------------
  ALTA_VOTACIO('Tunel Pirineus', 
               '10/12/2001', 
               '18/12/2001', 
               '22/12/2001', 
               'Jean Frances Spencer', 
               'Est d''acord amb el projecte de tnel per comunicar Catalunya amb Frana?. Si, No, Millor un altre lloc o Vot en blanc', 
               'http://www.europa.eu/consultes/regionals/tunel_pirineus', RSP);

  --- Opcions Votaci 22 (x4)
  ALTA_OPCIO('Tunel Pirineus', 'Si', RSP); 
  ALTA_OPCIO('Tunel Pirineus', 'No', RSP);
  ALTA_OPCIO('Tunel Pirineus', 'Millor un altre lloc', RSP);
  ALTA_OPCIO('Tunel Pirineus', 'Vot en blanc', RSP);

--------------------------------------------------------------------------------
---- Donem d'alta una quantitat suficient de ciutadans per poder fer proves ----
--------------------------------------------------------------------------------
  
  --- Obrim el cursor de Localitats
  OPEN C3;
  --- Definim una data inicial
  dn := '01/01/1993';
  LOOP    
    --- Fem un recorregut per totes elles
    FETCH C3 INTO v1, v2, v3;
    --- Si no hi han ms Localitats sortim
    EXIT WHEN C3%NOTFOUND;
    num := 0;
    LOOP
      --- 50 Ciutadans per Localitat
      EXIT WHEN num = 50;
      ALTA_CIUTADA(dn, v3, v2, v1, RSP); 
      num := num + 1;
      dn := dn - 5;
    END LOOP;
  END LOOP;
  --- Tanquem el cursor
  CLOSE C3;
  
--------------------------------------------------------------------------------
---- Inscrivim els Ciutadans als seus Censos -----------------------------------
--------------------------------------------------------------------------------
  
  --- Obrim el cursor de Ciutadans
  OPEN C4;
  --- Definim una data d'iscripci comuna
  dn := '01/01/1990';
  LOOP    
    --- Fem un recorregut per tots ells
    FETCH C4 INTO num;
    --- Si no hi han ms Ciutadans sortim
    EXIT WHEN C4%NOTFOUND;
      --- Inscrivim els Ciutadans
      ALTA_INSCRIPCIO(num, dn, RSP);     
  END LOOP;
  --- Tanquem el cursor
  CLOSE C4;
  
--------------------------------------------------------------------------------
---- Donem d'alta alguns Ciutadans sense inscriure als seus Censos -------------
--------------------------------------------------------------------------------
  
  --- Obrim el cursor de Localitats
  OPEN C3;
  --- Definim una data inicial
  dn := '01/01/1992';
  LOOP    
    --- Fem un recorregut per totes elles
    FETCH C3 INTO v1, v2, v3;
    --- Si no hi han ms Localitats sortim
    EXIT WHEN C3%NOTFOUND;
    num := 0;
    LOOP
      --- 5 Ciutadans per Localitat
      EXIT WHEN num = 5;
      ALTA_CIUTADA(dn, v3, v2, v1, RSP); 
      num := num + 1;
      dn := dn - 5;
    END LOOP;
  END LOOP;
  --- Tanquem el cursor
  CLOSE C3;
  
--------------------------------------------------------------------------------
------ Assignem els Censos a les Votacions -------------------------------------
--------------------------------------------------------------------------------
  --- Votaci 1
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Constitucio Europea'; 
  ALTA_ASSIGNACIO('Europa', 'Constitucio Europea', dn, RSP);
  --- Votaci 2
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Riera Premia'; 
  ALTA_ASSIGNACIO('Loc_ES_CAT_PREM', 'Riera Premia', dn, RSP);
  --- Votaci 3
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Europarlament 2010'; 
  ALTA_ASSIGNACIO('Est_ES', 'Europarlament 2010', dn, RSP);
  --- Votaci 4
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Corredor Mediterrani'; 
  ALTA_ASSIGNACIO('Est_ES', 'Corredor Mediterrani', dn, RSP);
  ALTA_ASSIGNACIO('Est_FR', 'Corredor Mediterrani', dn, RSP);
  ALTA_ASSIGNACIO('Est_PT', 'Corredor Mediterrani', dn, RSP);
  --- Votaci 5
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Corredor Central'; 
  ALTA_ASSIGNACIO('Est_ES', 'Corredor Central', dn, RSP);
  ALTA_ASSIGNACIO('Est_FR', 'Corredor Central', dn, RSP);
  ALTA_ASSIGNACIO('Est_PT', 'Corredor Central', dn, RSP);
  --- Votaci 6
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Depuradora Maresme'; 
  ALTA_ASSIGNACIO('Loc_ES_CAT_PREM', 'Depuradora Maresme', dn, RSP);
  ALTA_ASSIGNACIO('Loc_ES_CAT_MASN', 'Depuradora Maresme', dn, RSP);
  ALTA_ASSIGNACIO('Loc_ES_CAT_VILM', 'Depuradora Maresme', dn, RSP);
  --- Votaci 7
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Terme Barcelona'; 
  ALTA_ASSIGNACIO('Loc_ES_CAT_BARC', 'Terme Barcelona', dn, RSP);
  ALTA_ASSIGNACIO('Loc_ES_CAT_HOSP', 'Terme Barcelona', dn, RSP);
  --- Votaci 8
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Relacio ES FR'; 
  ALTA_ASSIGNACIO('Est_ES', 'Relacio ES FR', dn, RSP);
  ALTA_ASSIGNACIO('Est_FR', 'Relacio ES FR', dn, RSP);
  --- Votaci 9
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Relacio ES PT'; 
  ALTA_ASSIGNACIO('Est_ES', 'Relacio ES PT', dn, RSP);
  ALTA_ASSIGNACIO('Est_PT', 'Relacio ES PT', dn, RSP);
  --- Votaci 10
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Millora Platges'; 
  ALTA_ASSIGNACIO('Loc_ES_CAT_PREM', 'Millora Platges', dn, RSP);
  ALTA_ASSIGNACIO('Loc_ES_CAT_MASN', 'Millora Platges', dn, RSP);
  ALTA_ASSIGNACIO('Loc_ES_CAT_VILM', 'Millora Platges', dn, RSP);
  ALTA_ASSIGNACIO('Loc_ES_CAT_BARC', 'Millora Platges', dn, RSP);
  ALTA_ASSIGNACIO('Loc_FR_LRO_PERP', 'Millora Platges', dn, RSP);  
  --- Votaci 11
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Entrar Euro'; 
  ALTA_ASSIGNACIO('Europa', 'Entrar Euro', dn, RSP);
  --- Votaci 12
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Sortir Euro'; 
  ALTA_ASSIGNACIO('Europa', 'Sortir Euro', dn, RSP);
  --- Votaci 13
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Premia Mar i Dalt'; 
  ALTA_ASSIGNACIO('Loc_ES_CAT_PREM', 'Premia Mar i Dalt', dn, RSP);
  ALTA_ASSIGNACIO('Loc_ES_CAT_PRED', 'Premia Mar i Dalt', dn, RSP);
  --- Votaci 14
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Premia Mar Masnou'; 
  ALTA_ASSIGNACIO('Loc_ES_CAT_PREM', 'Premia Mar Masnou', dn, RSP);
  ALTA_ASSIGNACIO('Loc_ES_CAT_MASN', 'Premia Mar Masnou', dn, RSP);
  --- Votaci 15
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Obertura Atlantica'; 
  ALTA_ASSIGNACIO('Europa', 'Obertura Atlantica', dn, RSP);
  --- Votaci 16
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Atlantic Nord'; 
  ALTA_ASSIGNACIO('Europa', 'Atlantic Nord', dn, RSP);
  --- Votaci 17
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Banc Central'; 
  ALTA_ASSIGNACIO('Europa', 'Banc Central', dn, RSP);
  --- Votaci 18
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Eurobons';
  ALTA_ASSIGNACIO('Europa', 'Eurobons', dn, RSP);
  --- Votaci 19
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Mesures ES Crisis';
  ALTA_ASSIGNACIO('Est_ES', 'Mesures ES Crisis', dn, RSP);
  --- Votaci 20
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Mesures PT Crisis';
  ALTA_ASSIGNACIO('Est_PT', 'Mesures PT Crisis', dn, RSP);
  --- Votaci 21
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Ampliacio UE';
  ALTA_ASSIGNACIO('Europa', 'Ampliacio UE', dn, RSP);
  --- Votaci 22
  SELECT data_publicacio INTO dn FROM VOTACIO WHERE titol = 'Tunel Pirineus';
  ALTA_ASSIGNACIO('Reg_ES_CAT', 'Tunel Pirineus', dn, RSP);
  ALTA_ASSIGNACIO('Reg_FR_LRO', 'Tunel Pirineus', dn, RSP);
  
  --- Validem totes les transaccions.
  COMMIT;
END;
/

